@@ -1,44 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qrcparser.h"
 
-#include <utils/qtcassert.h>
+#include "filepath.h"
+#include "qtcassert.h"
+
+#include <qmljs/qmljstr.h> // Yes, the translations are still there
 
-#include <QCoreApplication>
 #include <QDir>
 #include <QDomDocument>
 #include <QFile>
 #include <QFileInfo>
 #include <QLocale>
 #include <QLoggingCategory>
-#include <QMultiHash>
-#include <QMutex>
-#include <QMutexLocker>
-#include <QSet>
-#include <QStringList>
+#include <QReadLocker>
+#include <QReadWriteLock>
+#include <QWriteLocker>
 
 static Q_LOGGING_CATEGORY(qrcParserLog, "qtc.qrcParser", QtWarningMsg)
 
@@ -48,44 +26,51 @@ namespace Internal {
 
 class QrcParserPrivate
 {
-    Q_DECLARE_TR_FUNCTIONS(QmlJS::QrcParser)
 public:
-    typedef QMap<QString,QStringList> SMap;
     QrcParserPrivate(QrcParser *q);
-    bool parseFile(const QString &path, const QString &contents);
-    QString firstFileAtPath(const QString &path, const QLocale &locale) const;
-    void collectFilesAtPath(const QString &path, QStringList *res, const QLocale *locale = nullptr) const;
+
+    bool parseFile(const FilePath &path, const QString &contents);
+    FilePath firstFileAtPath(const QString &path, const QLocale &locale) const;
+    void collectFilesAtPath(
+        const QString &path, FilePaths *res, const QLocale *locale = nullptr) const;
     bool hasDirAtPath(const QString &path, const QLocale *locale = nullptr) const;
-    void collectFilesInPath(const QString &path, QMap<QString,QStringList> *res, bool addDirs = false,
-                            const QLocale *locale = nullptr) const;
-    void collectResourceFilesForSourceFile(const QString &sourceFile, QStringList *res,
-                                           const QLocale *locale = nullptr) const;
+    void collectFilesInPath(
+        const QString &path,
+        QMap<QString, FilePaths> *res,
+        bool addDirs = false,
+        const QLocale *locale = nullptr) const;
+    void collectResourceFilesForSourceFile(
+        const FilePath &sourceFile, QStringList *res, const QLocale *locale = nullptr) const;
+    QrcParser::MatchResult longestReverseMatches(const QString &) const;
 
     QStringList errorMessages() const;
     QStringList languages() const;
+
 private:
     static QString fixPrefix(const QString &prefix);
     const QStringList allUiLanguages(const QLocale *locale) const;
 
-    SMap m_resources;
-    SMap m_files;
+    QMap<QString, FilePaths> m_resources;
+    QMap<QString, FilePaths> m_reverseResources;
+    QMap<FilePath, QStringList> m_files;
     QStringList m_languages;
     QStringList m_errorMessages;
 };
 
 class QrcCachePrivate
 {
-    Q_DECLARE_TR_FUNCTIONS(QmlJS::QrcCachePrivate)
 public:
     QrcCachePrivate(QrcCache *q);
-    QrcParser::Ptr addPath(const QString &path, const QString &contents);
-    void removePath(const QString &path);
-    QrcParser::Ptr updatePath(const QString &path, const QString &contents);
-    QrcParser::Ptr parsedPath(const QString &path);
+
+    QrcParser::Ptr addPath(const FilePath &path, const QString &contents);
+    void removePath(const FilePath &path);
+    QrcParser::Ptr updatePath(const FilePath &path, const QString &contents);
+    QrcParser::Ptr parsedPath(const FilePath &path);
     void clear();
+
 private:
-    QHash<QString, QPair<QrcParser::Ptr,int> > m_cache;
-    QMutex m_mutex;
+    QHash<FilePath, QPair<QrcParser::Ptr,int> > m_cache;
+    QReadWriteLock m_mutex;
 };
 } // namespace Internal
 
@@ -115,12 +100,12 @@ private:
  */
 
 /*!
-    \typedef QrcParser::Ptr
+    \typedef Utils::QrcParser::Ptr
     Represents pointers.
  */
 
 /*!
-    \typedef QrcParser::ConstPtr
+    \typedef Utils::QrcParser::ConstPtr
     Represents constant pointers.
 */
 
@@ -185,7 +170,7 @@ QrcParser::~QrcParser()
 
     \sa errorMessages(), parseQrcFile()
 */
-bool QrcParser::parseFile(const QString &path, const QString &contents)
+bool QrcParser::parseFile(const FilePath &path, const QString &contents)
 {
     return d->parseFile(path, contents);
 }
@@ -194,7 +179,7 @@ bool QrcParser::parseFile(const QString &path, const QString &contents)
     Returns the file system path of the first (active) file at the given QRC
     \a path and \a locale.
  */
-QString QrcParser::firstFileAtPath(const QString &path, const QLocale &locale) const
+FilePath QrcParser::firstFileAtPath(const QString &path, const QLocale &locale) const
 {
     return d->firstFileAtPath(path, locale);
 }
@@ -205,11 +190,16 @@ QString QrcParser::firstFileAtPath(const QString &path, const QLocale &locale) c
     If \a locale is null, all possible files are added. Otherwise, just
     the first one that matches the locale is added.
  */
-void QrcParser::collectFilesAtPath(const QString &path, QStringList *res, const QLocale *locale) const
+void QrcParser::collectFilesAtPath(const QString &path, FilePaths *res, const QLocale *locale) const
 {
     d->collectFilesAtPath(path, res, locale);
 }
 
+QrcParser::MatchResult QrcParser::longestReverseMatches(const QString &p) const
+{
+    return d->longestReverseMatches(p);
+}
+
 /*!
     Returns \c true if \a path is a non-empty directory and matches \a locale.
 
@@ -230,7 +220,7 @@ bool QrcParser::hasDirAtPath(const QString &path, const QLocale *locale) const
     If \a locale is null, all possible files are added. Otherwise, just the
     first file with a matching the locale is added.
  */
-void QrcParser::collectFilesInPath(const QString &path, QMap<QString,QStringList> *res, bool addDirs,
+void QrcParser::collectFilesInPath(const QString &path, QMap<QString, FilePaths> *res, bool addDirs,
                                    const QLocale *locale) const
 {
     d->collectFilesInPath(path, res, addDirs, locale);
@@ -242,8 +232,8 @@ void QrcParser::collectFilesInPath(const QString &path, QMap<QString,QStringList
     If \a locale is null, all possible files are added. Otherwise, just
     the first file with a matching the locale is added.
  */
-void QrcParser::collectResourceFilesForSourceFile(const QString &sourceFile, QStringList *res,
-                                                  const QLocale *locale) const
+void QrcParser::collectResourceFilesForSourceFile(
+    const FilePath &sourceFile, QStringList *res, const QLocale *locale) const
 {
     d->collectResourceFilesForSourceFile(sourceFile, res, locale);
 }
@@ -277,7 +267,7 @@ bool QrcParser::isValid() const
 /*!
     Returns the \a contents of the QRC file at \a path.
 */
-QrcParser::Ptr QrcParser::parseQrcFile(const QString &path, const QString &contents)
+QrcParser::Ptr QrcParser::parseQrcFile(const FilePath &path, const QString &contents)
 {
     Ptr res(new QrcParser);
     if (!path.isEmpty())
@@ -317,7 +307,7 @@ QrcCache::~QrcCache()
 
     \sa QrcParser::errorMessages(), QrcParser::parseQrcFile()
 */
-QrcParser::ConstPtr QrcCache::addPath(const QString &path, const QString &contents)
+QrcParser::ConstPtr QrcCache::addPath(const FilePath &path, const QString &contents)
 {
     return d->addPath(path, contents);
 }
@@ -325,7 +315,7 @@ QrcParser::ConstPtr QrcCache::addPath(const QString &path, const QString &conten
 /*!
     Removes \a path from the cache.
 */
-void QrcCache::removePath(const QString &path)
+void QrcCache::removePath(const FilePath &path)
 {
     d->removePath(path);
 }
@@ -333,7 +323,7 @@ void QrcCache::removePath(const QString &path)
 /*!
     Reparses the QRC file at \a path and returns the \a contents of the file.
 */
-QrcParser::ConstPtr QrcCache::updatePath(const QString &path, const QString &contents)
+QrcParser::ConstPtr QrcCache::updatePath(const FilePath &path, const QString &contents)
 {
     return d->updatePath(path, contents);
 }
@@ -341,7 +331,7 @@ QrcParser::ConstPtr QrcCache::updatePath(const QString &path, const QString &con
 /*!
     Returns the cached QRC parser for the QRC file at \a path.
 */
-QrcParser::ConstPtr QrcCache::parsedPath(const QString &path)
+QrcParser::ConstPtr QrcCache::parsedPath(const FilePath &path)
 {
     return d->parsedPath(path);
 }
@@ -361,14 +351,14 @@ namespace Internal {
 QrcParserPrivate::QrcParserPrivate(QrcParser *)
 { }
 
-bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
+bool QrcParserPrivate::parseFile(const FilePath &path, const QString &contents)
 {
     QDomDocument doc;
-    QDir baseDir(QFileInfo(path).path());
+    FilePath baseDir = path.parentDir();
 
     if (contents.isEmpty()) {
         // Regular file
-        QFile file(path);
+        QFile file(path.toFSPathString());
         if (!file.open(QIODevice::ReadOnly)) {
             m_errorMessages.append(file.errorString());
             return false;
@@ -377,7 +367,7 @@ bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
         QString error_msg;
         int error_line, error_col;
         if (!doc.setContent(&file, &error_msg, &error_line, &error_col)) {
-            m_errorMessages.append(tr("XML error on line %1, col %2: %3")
+            m_errorMessages.append(QmlJS::Tr::tr("XML error on line %1, col %2: %3")
                                    .arg(error_line).arg(error_col).arg(error_msg));
             return false;
         }
@@ -386,7 +376,7 @@ bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
         QString error_msg;
         int error_line, error_col;
         if (!doc.setContent(contents, &error_msg, &error_line, &error_col)) {
-            m_errorMessages.append(tr("XML error on line %1, col %2: %3")
+            m_errorMessages.append(QmlJS::Tr::tr("XML error on line %1, col %2: %3")
                                    .arg(error_line).arg(error_col).arg(error_msg));
             return false;
         }
@@ -394,7 +384,7 @@ bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
 
     QDomElement root = doc.firstChildElement(QLatin1String("RCC"));
     if (root.isNull()) {
-        m_errorMessages.append(tr("The <RCC> root element is missing."));
+        m_errorMessages.append(QmlJS::Tr::tr("The <RCC> root element is missing."));
         return false;
     }
 
@@ -410,15 +400,21 @@ bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
         for (; !felt.isNull(); felt = felt.nextSiblingElement(QLatin1String("file"))) {
             const QString fileName = felt.text();
             const QString alias = felt.attribute(QLatin1String("alias"));
-            QString filePath = baseDir.absoluteFilePath(fileName);
+            FilePath filePath = baseDir.resolvePath(fileName);
             QString accessPath;
             if (!alias.isEmpty())
                 accessPath = language + prefix + alias;
             else
                 accessPath = language + prefix + fileName;
-            QStringList &resources = m_resources[accessPath];
-            if (!resources.contains(filePath))
+            FilePaths &resources = m_resources[accessPath];
+            if (!resources.contains(filePath)) {
                 resources.append(filePath);
+                QString reversePath(accessPath);
+                std::reverse(reversePath.begin(), reversePath.end());
+                if (!reversePath.endsWith('/'))
+                    reversePath.append('/');
+                m_reverseResources[reversePath].append(filePath);
+            }
             QStringList &files = m_files[filePath];
             if (!files.contains(accessPath))
                 files.append(accessPath);
@@ -428,26 +424,26 @@ bool QrcParserPrivate::parseFile(const QString &path, const QString &contents)
 }
 
 // path is assumed to be a normalized absolute path
-QString QrcParserPrivate::firstFileAtPath(const QString &path, const QLocale &locale) const
+FilePath QrcParserPrivate::firstFileAtPath(const QString &path, const QLocale &locale) const
 {
     QTC_CHECK(path.startsWith(QLatin1Char('/')));
     for (const QString &language : allUiLanguages(&locale)) {
         if (m_languages.contains(language)) {
-            SMap::const_iterator res = m_resources.find(language + path);
+            auto res = m_resources.find(language + path);
             if (res != m_resources.end())
                 return res.value().at(0);
         }
     }
-    return QString();
+    return {};
 }
 
-void QrcParserPrivate::collectFilesAtPath(const QString &path, QStringList *files,
+void QrcParserPrivate::collectFilesAtPath(const QString &path, FilePaths *files,
                                           const QLocale *locale) const
 {
     QTC_CHECK(path.startsWith(QLatin1Char('/')));
     for (const QString &language : allUiLanguages(locale)) {
         if (m_languages.contains(language)) {
-            SMap::const_iterator res = m_resources.find(language + path);
+            auto res = m_resources.find(language + path);
             if (res != m_resources.end())
                 (*files) << res.value();
         }
@@ -462,7 +458,7 @@ bool QrcParserPrivate::hasDirAtPath(const QString &path, const QLocale *locale)
     for (const QString &language : allUiLanguages(locale)) {
         if (m_languages.contains(language)) {
             QString key = language + path;
-            SMap::const_iterator res = m_resources.lowerBound(key);
+            auto res = m_resources.lowerBound(key);
             if (res != m_resources.end() && res.key().startsWith(key))
                 return true;
         }
@@ -470,29 +466,29 @@ bool QrcParserPrivate::hasDirAtPath(const QString &path, const QLocale *locale)
     return false;
 }
 
-void QrcParserPrivate::collectFilesInPath(const QString &path, QMap<QString,QStringList> *contents,
+void QrcParserPrivate::collectFilesInPath(const QString &path, QMap<QString, FilePaths> *contents,
                                           bool addDirs, const QLocale *locale) const
 {
     QTC_CHECK(path.startsWith(QLatin1Char('/')));
     QTC_CHECK(path.endsWith(QLatin1Char('/')));
-    SMap::const_iterator end = m_resources.end();
+    auto end = m_resources.end();
     for (const QString &language : allUiLanguages(locale)) {
         QString key = language + path;
-        SMap::const_iterator res = m_resources.lowerBound(key);
+        auto res = m_resources.lowerBound(key);
         while (res != end && res.key().startsWith(key)) {
             const QString &actualKey = res.key();
             int endDir = actualKey.indexOf(QLatin1Char('/'), key.size());
             if (endDir == -1) {
                 QString fileName = res.key().right(res.key().size()-key.size());
-                QStringList &els = (*contents)[fileName];
-                for (const QString &val : res.value())
+                FilePaths &els = (*contents)[fileName];
+                for (const FilePath &val : res.value())
                     if (!els.contains(val))
                         els << val;
                 ++res;
             } else {
                 QString dirName = res.key().mid(key.size(), endDir - key.size() + 1);
                 if (addDirs)
-                    contents->insert(dirName, QStringList());
+                    contents->insert(dirName, {});
                 QString key2 = key + dirName;
                 do {
                     ++res;
@@ -502,14 +498,14 @@ void QrcParserPrivate::collectFilesInPath(const QString &path, QMap<QString,QStr
     }
 }
 
-void QrcParserPrivate::collectResourceFilesForSourceFile(const QString &sourceFile,
+void QrcParserPrivate::collectResourceFilesForSourceFile(const FilePath &sourceFile,
                                                          QStringList *results,
                                                          const QLocale *locale) const
 {
     // TODO: use FileName from fileutils for file paths
 
     const QStringList langs = allUiLanguages(locale);
-    SMap::const_iterator file = m_files.find(sourceFile);
+    auto file = m_files.find(sourceFile);
     if (file == m_files.end())
         return;
     for (const QString &resource : file.value()) {
@@ -520,6 +516,37 @@ void QrcParserPrivate::collectResourceFilesForSourceFile(const QString &sourceFi
     }
 }
 
+QrcParser::MatchResult QrcParserPrivate::longestReverseMatches(const QString &reversePath) const
+{
+    QrcParser::MatchResult res;
+    if (reversePath.length() == 1)
+        return res;
+    auto lastMatch = m_reverseResources.end();
+    qsizetype matchedUntil = 0;
+    for (qsizetype i = 1, j = 0; i < reversePath.size(); i = j + 1) {
+        j = reversePath.indexOf(u'/', i);
+        if (j == -1)
+            j = reversePath.size() - 1;
+        auto match = m_reverseResources.lowerBound(reversePath.mid(0, j + 1));
+        QString pNow = reversePath.left(j + 1);
+        if (match == m_reverseResources.end() || match.key().left(j + 1) != pNow)
+            break;
+        ++res.matchDepth;
+        matchedUntil = j + 1;
+        lastMatch = match;
+    }
+    res.reversedPaths.clear();
+    res.sourceFiles.clear();
+    for (auto it = lastMatch; it != m_reverseResources.end()
+                              && it.key().left(matchedUntil) == reversePath.left(matchedUntil);
+         ++it) {
+        res.reversedPaths.append(it.key());
+        for (const FilePath &filePath : it.value())
+            res.sourceFiles.append(filePath);
+    }
+    return res;
+}
+
 QStringList QrcParserPrivate::errorMessages() const
 {
     return m_errorMessages;
@@ -551,18 +578,21 @@ const QStringList QrcParserPrivate::allUiLanguages(const QLocale *locale) const
 {
     if (!locale)
         return languages();
-    QStringList langs = locale->uiLanguages();
-    foreach (const QString &language, langs) { // qt4 support
-        if (language.contains(QLatin1Char('_')) || language.contains(QLatin1Char('-'))) {
-            QStringList splits = QString(language).replace(QLatin1Char('_'), QLatin1Char('-'))
-                    .split(QLatin1Char('-'));
-            if (splits.size() > 1 && !langs.contains(splits.at(0)))
-                langs.append(splits.at(0));
+    bool hasEmptyString = false;
+    const QStringList langs = locale->uiLanguages();
+    QStringList allLangs = langs;
+    for (const QString &language : langs) {
+        if (language.isEmpty())
+            hasEmptyString = true;
+        else if (language.contains('_') || language.contains('-')) {
+            const QStringList splits = QString(language).replace('_', '-').split('-');
+            if (splits.size() > 1 && !allLangs.contains(splits.at(0)))
+                allLangs.append(splits.at(0));
         }
     }
-    if (!langs.contains(QString()))
-        langs.append(QString());
-    return langs;
+    if (!hasEmptyString)
+        allLangs.append(QString());
+    return allLangs;
 }
 
 // ----------------
@@ -570,11 +600,11 @@ const QStringList QrcParserPrivate::allUiLanguages(const QLocale *locale) const
 QrcCachePrivate::QrcCachePrivate(QrcCache *)
 { }
 
-QrcParser::Ptr QrcCachePrivate::addPath(const QString &path, const QString &contents)
+QrcParser::Ptr QrcCachePrivate::addPath(const FilePath &path, const QString &contents)
 {
     QPair<QrcParser::Ptr,int> currentValue;
     {
-        QMutexLocker l(&m_mutex);
+        QWriteLocker l(&m_mutex);
         currentValue = m_cache.value(path, {QrcParser::Ptr(nullptr), 0});
         currentValue.second += 1;
         if (currentValue.second > 1) {
@@ -586,9 +616,9 @@ QrcParser::Ptr QrcCachePrivate::addPath(const QString &path, const QString &cont
     if (!newParser->isValid())
         qCWarning(qrcParserLog) << "adding invalid qrc " << path << " to the cache:" << newParser->errorMessages();
     {
-        QMutexLocker l(&m_mutex);
-        QPair<QrcParser::Ptr,int> currentValue = m_cache.value(path, {QrcParser::Ptr(nullptr), 0});
-        if (currentValue.first.isNull())
+        QWriteLocker l(&m_mutex);
+        QPair<QrcParser::Ptr, int> currentValue = m_cache.value(path, {{}, 0});
+        if (!currentValue.first)
             currentValue.first = newParser;
         currentValue.second += 1;
         m_cache.insert(path, currentValue);
@@ -596,11 +626,11 @@ QrcParser::Ptr QrcCachePrivate::addPath(const QString &path, const QString &cont
     }
 }
 
-void QrcCachePrivate::removePath(const QString &path)
+void QrcCachePrivate::removePath(const FilePath &path)
 {
     QPair<QrcParser::Ptr,int> currentValue;
     {
-        QMutexLocker l(&m_mutex);
+        QWriteLocker l(&m_mutex);
         currentValue = m_cache.value(path, {QrcParser::Ptr(nullptr), 0});
         if (currentValue.second == 1) {
             m_cache.remove(path);
@@ -613,11 +643,11 @@ void QrcCachePrivate::removePath(const QString &path)
     }
 }
 
-QrcParser::Ptr QrcCachePrivate::updatePath(const QString &path, const QString &contents)
+QrcParser::Ptr QrcCachePrivate::updatePath(const FilePath &path, const QString &contents)
 {
     QrcParser::Ptr newParser = QrcParser::parseQrcFile(path, contents);
     {
-        QMutexLocker l(&m_mutex);
+        QWriteLocker l(&m_mutex);
         QPair<QrcParser::Ptr,int> currentValue = m_cache.value(path, {QrcParser::Ptr(nullptr), 0});
         currentValue.first = newParser;
         if (currentValue.second == 0)
@@ -627,18 +657,18 @@ QrcParser::Ptr QrcCachePrivate::updatePath(const QString &path, const QString &c
     }
 }
 
-QrcParser::Ptr QrcCachePrivate::parsedPath(const QString &path)
+QrcParser::Ptr QrcCachePrivate::parsedPath(const FilePath &path)
 {
-    QMutexLocker l(&m_mutex);
+    QReadLocker l(&m_mutex);
     QPair<QrcParser::Ptr,int> currentValue = m_cache.value(path, {QrcParser::Ptr(nullptr), 0});
     return currentValue.first;
 }
 
 void QrcCachePrivate::clear()
 {
-    QMutexLocker l(&m_mutex);
+    QWriteLocker l(&m_mutex);
     m_cache.clear();
 }
 
 } // namespace Internal
-} // namespace QmlJS
+} // namespace Utils
