@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "clangpreprocessorassistproposalitem.h"
 
@@ -50,10 +28,11 @@ bool ClangPreprocessorAssistProposalItem::implicitlyApplies() const
     return true;
 }
 
-void ClangPreprocessorAssistProposalItem::apply(TextEditor::TextDocumentManipulatorInterface &manipulator,
+void ClangPreprocessorAssistProposalItem::apply(TextEditor::TextEditorWidget *editorWidget,
                                                 int basePosition) const
 {
     // TODO move in an extra class under tests
+    QTC_ASSERT(editorWidget, return);
 
     QString textToBeInserted = text();
 
@@ -73,12 +52,13 @@ void ClangPreprocessorAssistProposalItem::apply(TextEditor::TextDocumentManipula
         extraCharacters += m_typedCharacter;
 
     // Avoid inserting characters that are already there
-    const int endsPosition = manipulator.positionAt(TextEditor::EndOfLinePosition);
-    const QString existingText = manipulator.textAt(manipulator.currentPosition(), endsPosition - manipulator.currentPosition());
+    QTextCursor c = editorWidget->textCursor();
+    c.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+    const QString existingText = c.selectedText();
     int existLength = 0;
     if (!existingText.isEmpty()) {
         // Calculate the exist length in front of the extra chars
-        existLength = textToBeInserted.length() - (manipulator.currentPosition() - basePosition);
+        existLength = textToBeInserted.length() - (editorWidget->position() - basePosition);
         while (!existingText.startsWith(textToBeInserted.right(existLength))) {
             if (--existLength == 0)
                 break;
@@ -86,7 +66,7 @@ void ClangPreprocessorAssistProposalItem::apply(TextEditor::TextDocumentManipula
     }
     for (int i = 0; i < extraCharacters.length(); ++i) {
         const QChar a = extraCharacters.at(i);
-        const QChar b = manipulator.characterAt(manipulator.currentPosition() + i + existLength);
+        const QChar b = editorWidget->characterAt(editorWidget->position() + i + existLength);
         if (a == b)
             ++extraLength;
         else
@@ -96,9 +76,9 @@ void ClangPreprocessorAssistProposalItem::apply(TextEditor::TextDocumentManipula
     textToBeInserted += extraCharacters;
 
     // Insert the remainder of the name
-    const int length = manipulator.currentPosition() - basePosition + existLength + extraLength;
+    const int length = editorWidget->position() - basePosition + existLength + extraLength;
 
-    manipulator.replace(basePosition, length, textToBeInserted);
+    editorWidget->replace(basePosition, length, textToBeInserted);
 }
 
 void ClangPreprocessorAssistProposalItem::setText(const QString &text)
@@ -131,6 +111,11 @@ QString ClangPreprocessorAssistProposalItem::detail() const
     return QString();
 }
 
+Qt::TextFormat ClangPreprocessorAssistProposalItem::detailFormat() const
+{
+    return Qt::RichText;
+}
+
 bool ClangPreprocessorAssistProposalItem::isSnippet() const
 {
     return false;
