@@ -3,21 +3,25 @@
 
 #include "fileapiparser.h"
 
+#include "cmakeprocess.h"
+#include "cmakeprojectconstants.h"
 #include "cmakeprojectmanagertr.h"
 
-#include <app/app_version.h>
 #include <coreplugin/messagemanager.h>
 #include <projectexplorer/rawprojectpart.h>
 
 #include <utils/algorithm.h>
+#include <utils/filepath.h>
 #include <utils/qtcassert.h>
 
+#include <QGuiApplication>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
 #include <QPromise>
 
+using namespace ProjectExplorer;
 using namespace Utils;
 
 namespace CMakeProjectManager::Internal {
@@ -29,8 +33,6 @@ const char CMAKE_RELATIVE_QUERY_PATH[] = ".cmake/api/v1/query";
 
 static Q_LOGGING_CATEGORY(cmakeFileApi, "qtc.cmake.fileApi", QtWarningMsg);
 
-const QStringList CMAKE_QUERY_FILENAMES = {"cache-v2", "codemodel-v2", "cmakeFiles-v1"};
-
 // --------------------------------------------------------------------
 // Helper:
 // --------------------------------------------------------------------
@@ -43,9 +45,9 @@ FilePath FileApiParser::cmakeReplyDirectory(const FilePath &buildDirectory)
 static void reportFileApiSetupFailure()
 {
     Core::MessageManager::writeFlashing(
-        Tr::tr("Failed to set up CMake file API support. %1 cannot "
-               "extract project information.")
-            .arg(Core::Constants::IDE_DISPLAY_NAME));
+        addCMakePrefix(Tr::tr("Failed to set up CMake file API support. %1 cannot "
+                              "extract project information.")
+                           .arg(QGuiApplication::applicationDisplayName())));
 }
 
 static std::pair<int, int> cmakeVersion(const QJsonObject &obj)
@@ -76,7 +78,7 @@ static QJsonDocument readJsonFile(const FilePath &filePath)
     qCDebug(cmakeFileApi) << "readJsonFile:" << filePath;
     QTC_ASSERT(!filePath.isEmpty(), return {});
 
-    const expected_str<QByteArray> contents = filePath.fileContents();
+    const Result<QByteArray> contents = filePath.fileContents();
     if (!contents)
         return {};
     const QJsonDocument doc = QJsonDocument::fromJson(*contents);
@@ -90,7 +92,7 @@ std::vector<int> indexList(const QJsonValue &v)
     std::vector<int> result;
     result.reserve(static_cast<size_t>(indexList.count()));
 
-    for (const QJsonValue &v : indexList) {
+    for (const auto &v : indexList) {
         result.push_back(v.toInt(-1));
     }
     return result;
@@ -138,7 +140,7 @@ static ReplyFileContents readReplyFile(const FilePath &filePath, QString &errorM
     bool hadInvalidObject = false;
     {
         const QJsonArray objects = rootObject.value("objects").toArray();
-        for (const QJsonValue &v : objects) {
+        for (const auto &v : objects) {
             const QJsonObject object = v.toObject();
             {
                 ReplyObject r;
@@ -177,7 +179,7 @@ static CMakeConfig readCacheFile(const FilePath &cacheFile, QString &errorMessag
     }
 
     const QJsonArray entries = root.value("entries").toArray();
-    for (const QJsonValue &v : entries) {
+    for (const auto &v : entries) {
         CMakeConfigItem item;
 
         const QJsonObject entry = v.toObject();
@@ -189,12 +191,12 @@ static CMakeConfig readCacheFile(const FilePath &cacheFile, QString &errorMessag
 
         {
             const QJsonArray properties = entry.value("properties").toArray();
-            for (const QJsonValue &v : properties) {
+            for (const auto &v : properties) {
                 const QJsonObject prop = v.toObject();
                 auto nv = nameValue(prop);
                 if (nv.first == "ADVANCED") {
                     const auto boolValue = CMakeConfigItem::toBool(nv.second);
-                    item.isAdvanced = boolValue.has_value() && boolValue.value();
+                    item.isAdvanced = boolValue.has_value() && *boolValue;
                 } else if (nv.first == "HELPSTRING") {
                     item.documentation = nv.second.toUtf8();
                 } else if (nv.first == "STRINGS") {
@@ -222,14 +224,14 @@ static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesF
     }
 
     const QJsonArray inputs = root.value("inputs").toArray();
-    for (const QJsonValue &v : inputs) {
+    for (const auto &v : inputs) {
         CMakeFileInfo info;
         const QJsonObject input = v.toObject();
         info.path = cmakeFilesFile.withNewPath(input.value("path").toString());
 
         info.isCMake = input.value("isCMake").toBool();
         const QString filename = info.path.fileName();
-        info.isCMakeListsDotTxt = (filename.compare("CMakeLists.txt",
+        info.isCMakeListsDotTxt = (filename.compare(Constants::CMAKE_LISTS_TXT,
                                                     HostOsInfo::fileNameCaseSensitivity())
                                    == 0);
 
@@ -243,7 +245,7 @@ static std::vector<CMakeFileInfo> readCMakeFilesFile(const FilePath &cmakeFilesF
 
 // Codemodel file:
 
-std::vector<Directory> extractDirectories(const QJsonArray &directories, QString &errorMessage)
+std::vector<DirectoryInfo> extractDirectories(const QJsonArray &directories, QString &errorMessage)
 {
     if (directories.isEmpty()) {
         errorMessage = Tr::tr(
@@ -251,15 +253,15 @@ std::vector<Directory> extractDirectories(const QJsonArray &directories, QString
         return {};
     }
 
-    std::vector<Directory> result;
-    for (const QJsonValue &v : directories) {
+    std::vector<DirectoryInfo> result;
+    for (const auto &v : directories) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
             errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty directory object.");
             continue;
         }
-        Directory dir;
+        DirectoryInfo dir;
         dir.sourcePath = obj.value("source").toString();
         dir.buildPath = obj.value("build").toString();
         dir.parent = obj.value("parentIndex").toInt(-1);
@@ -273,7 +275,7 @@ std::vector<Directory> extractDirectories(const QJsonArray &directories, QString
     return result;
 }
 
-static std::vector<Project> extractProjects(const QJsonArray &projects, QString &errorMessage)
+static std::vector<ProjectInfo> extractProjects(const QJsonArray &projects, QString &errorMessage)
 {
     if (projects.isEmpty()) {
         errorMessage = Tr::tr(
@@ -281,8 +283,8 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
         return {};
     }
 
-    std::vector<Project> result;
-    for (const QJsonValue &v : projects) {
+    std::vector<ProjectInfo> result;
+    for (const auto &v : projects) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
             qCDebug(cmakeFileApi) << "Empty project skipped!";
@@ -290,7 +292,7 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
                 "Invalid codemodel file generated by CMake: Empty project object.");
             continue;
         }
-        Project project;
+        ProjectInfo project;
         project.name = obj.value("name").toString();
         project.parent = obj.value("parentIndex").toInt(-1);
         project.children = indexList(obj.value("childIndexes"));
@@ -310,17 +312,17 @@ static std::vector<Project> extractProjects(const QJsonArray &projects, QString
     return result;
 }
 
-static std::vector<Target> extractTargets(const QJsonArray &targets, QString &errorMessage)
+static std::vector<TargetInfo> extractTargets(const QJsonArray &targets, QString &errorMessage)
 {
-    std::vector<Target> result;
-    for (const QJsonValue &v : targets) {
+    std::vector<TargetInfo> result;
+    for (const auto &v : targets) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
             errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty target object.");
             continue;
         }
-        Target target;
+        TargetInfo target;
         target.name = obj.value("name").toString();
         target.id = obj.value("id").toString();
         target.directory = obj.value("directoryIndex").toInt(-1);
@@ -339,14 +341,14 @@ static std::vector<Target> extractTargets(const QJsonArray &targets, QString &er
     return result;
 }
 
-static bool validateIndexes(const Configuration &config)
+static bool validateIndexes(const ConfigurationInfo &config)
 {
     const int directoryCount = static_cast<int>(config.directories.size());
     const int projectCount = static_cast<int>(config.projects.size());
     const int targetCount = static_cast<int>(config.targets.size());
 
     int topLevelCount = 0;
-    for (const Directory &d : config.directories) {
+    for (const DirectoryInfo &d : config.directories) {
         if (d.parent == -1)
             ++topLevelCount;
 
@@ -378,7 +380,7 @@ static bool validateIndexes(const Configuration &config)
     }
 
     topLevelCount = 0;
-    for (const Project &p : config.projects) {
+    for (const ProjectInfo &p : config.projects) {
         if (p.parent == -1)
             ++topLevelCount;
 
@@ -410,7 +412,7 @@ static bool validateIndexes(const Configuration &config)
         return false;
     }
 
-    for (const Target &t : config.targets) {
+    for (const TargetInfo &t : config.targets) {
         if (t.directory < 0 || t.directory >= directoryCount) {
             qCWarning(cmakeFileApi)
                 << "Target" << t.name << ": directory index" << t.directory << "is broken.";
@@ -425,7 +427,7 @@ static bool validateIndexes(const Configuration &config)
     return true;
 }
 
-static std::vector<Configuration> extractConfigurations(const QJsonArray &configs,
+static std::vector<ConfigurationInfo> extractConfigurations(const QJsonArray &configs,
                                                         QString &errorMessage)
 {
     if (configs.isEmpty()) {
@@ -434,15 +436,15 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
         return {};
     }
 
-    std::vector<FileApiDetails::Configuration> result;
-    for (const QJsonValue &v : configs) {
+    std::vector<FileApiDetails::ConfigurationInfo> result;
+    for (const auto &v : configs) {
         const QJsonObject obj = v.toObject();
         if (obj.isEmpty()) {
             errorMessage = Tr::tr(
                 "Invalid codemodel file generated by CMake: Empty configuration object.");
             continue;
         }
-        Configuration config;
+        ConfigurationInfo config;
         config.name = obj.value("name").toString();
 
         config.directories = extractDirectories(obj.value("directories").toArray(), errorMessage);
@@ -460,7 +462,7 @@ static std::vector<Configuration> extractConfigurations(const QJsonArray &config
     return result;
 }
 
-static std::vector<Configuration> readCodemodelFile(const FilePath &codemodelFile,
+static std::vector<ConfigurationInfo> readCodemodelFile(const FilePath &codemodelFile,
                                                     QString &errorMessage)
 {
     const QJsonDocument doc = readJsonFile(codemodelFile);
@@ -486,27 +488,26 @@ static std::vector<FileApiDetails::FragmentInfo> extractFragments(const QJsonObj
     });
 }
 
-static void addIncludeInfo(std::vector<IncludeInfo> *includes,
+static void addIncludeInfo(const FilePath &replyDir,
+                           std::vector<IncludeInfo> *includes,
                            const QJsonObject &compileGroups,
                            const QString &section)
 {
     const std::vector<IncludeInfo> add
-        = transform<std::vector>(compileGroups.value(section).toArray(), [](const QJsonValue &v) {
+        = transform<std::vector>(compileGroups.value(section).toArray(), [replyDir](const QJsonValue &v) {
               const QJsonObject i = v.toObject();
-              const QString path = i.value("path").toString();
+              const FilePath path = replyDir.withNewPath(i.value("path").toString());
               const bool isSystem = i.value("isSystem").toBool();
-              const ProjectExplorer::HeaderPath hp(path,
-                                                   isSystem
-                                                       ? ProjectExplorer::HeaderPathType::System
-                                                       : ProjectExplorer::HeaderPathType::User);
+              const HeaderPath hp(path, isSystem ? HeaderPathType::System : HeaderPathType::User);
 
-              return IncludeInfo{ProjectExplorer::RawProjectPart::frameworkDetectionHeuristic(hp),
+              return IncludeInfo{RawProjectPart::frameworkDetectionHeuristic(hp),
                                  i.value("backtrace").toInt(-1)};
           });
     std::copy(add.cbegin(), add.cend(), std::back_inserter(*includes));
 }
 
-static TargetDetails extractTargetDetails(const QJsonObject &root, QString &errorMessage)
+static TargetDetails extractTargetDetails
+    (const FilePath &replyDir, const QJsonObject &root, QString &errorMessage)
 {
     TargetDetails t;
     t.name = root.value("name").toString();
@@ -525,8 +526,8 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
     }
     {
         const QJsonObject paths = root.value("paths").toObject();
-        t.sourceDir = FilePath::fromString(paths.value("source").toString());
-        t.buildDir = FilePath::fromString(paths.value("build").toString());
+        t.sourceDir = replyDir.withNewPath(paths.value("source").toString());
+        t.buildDir = replyDir.withNewPath(paths.value("build").toString());
     }
     t.nameOnDisk = root.value("nameOnDisk").toString();
     {
@@ -581,9 +582,9 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
     }
     {
         const QJsonArray sources = root.value("sources").toArray();
-        t.sources = transform<std::vector>(sources, [](const QJsonValue &v) {
+        t.sources = transform<std::vector>(sources, [replyDir](const QJsonValue &v) {
             const QJsonObject o = v.toObject();
-            return SourceInfo{o.value("path").toString(),
+            return SourceInfo{replyDir.withNewPath(o.value("path").toString()),
                               o.value("compileGroupIndex").toInt(-1),
                               o.value("sourceGroupIndex").toInt(-1),
                               o.value("backtrace").toInt(-1),
@@ -599,12 +600,12 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
     }
     {
         const QJsonArray compileGroups = root.value("compileGroups").toArray();
-        t.compileGroups = transform<std::vector>(compileGroups, [](const QJsonValue &v) {
+        t.compileGroups = transform<std::vector>(compileGroups, [replyDir](const QJsonValue &v) {
             const QJsonObject o = v.toObject();
             std::vector<IncludeInfo> includes;
-            addIncludeInfo(&includes, o, "includes");
+            addIncludeInfo(replyDir, &includes, o, "includes");
             // new in CMake 3.27+:
-            addIncludeInfo(&includes, o, "frameworks");
+            addIncludeInfo(replyDir, &includes, o, "frameworks");
             return CompileInfo{
                 transform<std::vector>(o.value("sourceIndexes").toArray(),
                                        [](const QJsonValue &v) { return v.toInt(-1); }),
@@ -619,8 +620,7 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
                                        [](const QJsonValue &v) {
                                            const QJsonObject d = v.toObject();
                                            return DefineInfo{
-                                               ProjectExplorer::Macro::fromKeyValue(
-                                                   d.value("define").toString()),
+                                               Macro::fromKeyValue(d.value("define").toString()),
                                                d.value("backtrace").toInt(-1),
                                            };
                                        }),
@@ -648,6 +648,19 @@ static TargetDetails extractTargetDetails(const QJsonObject &root, QString &erro
                                                             };
                                                         });
     }
+    {
+        const QJsonArray launchers = root.value("launchers").toArray();
+        if (launchers.size() > 0) {
+            t.launcherInfos = transform<QList>(launchers, [](const QJsonValue &v) {
+                const QJsonObject o = v.toObject();
+                QStringList arguments;
+                for (const QJsonValue &arg : o.value("arguments").toArray())
+                    arguments.append(arg.toString());
+                FilePath command = FilePath::fromString(o.value("command").toString());
+                return LauncherInfo { o.value("type").toString(), command, arguments };
+            });
+        }
+    }
 
     return t;
 }
@@ -765,12 +778,12 @@ static bool validateTargetDetails(const TargetDetails &t)
     return true;
 }
 
-static TargetDetails readTargetFile(const FilePath &targetFile, QString &errorMessage)
+static TargetDetails readTargetFile(const FilePath &replyDir, const FilePath &targetFile, QString &errorMessage)
 {
     const QJsonDocument doc = readJsonFile(targetFile);
     const QJsonObject root = doc.object();
 
-    TargetDetails result = extractTargetDetails(root, errorMessage);
+    TargetDetails result = extractTargetDetails(replyDir, root, errorMessage);
     if (errorMessage.isEmpty() && !validateTargetDetails(result)) {
         errorMessage = Tr::tr(
             "Invalid target file generated by CMake: Broken indexes in target details.");
@@ -821,11 +834,11 @@ bool FileApiParser::setupCMakeFileApi(const FilePath &buildDirectory)
     return true;
 }
 
-static QStringList uniqueTargetFiles(const Configuration &config)
+static QStringList uniqueTargetFiles(const ConfigurationInfo &config)
 {
     QSet<QString> knownIds;
     QStringList files;
-    for (const Target &t : config.targets) {
+    for (const TargetInfo &t : config.targets) {
         const int knownCount = knownIds.count();
         knownIds.insert(t.id);
         if (knownIds.count() > knownCount) {
@@ -837,6 +850,7 @@ static QStringList uniqueTargetFiles(const Configuration &config)
 
 FileApiData FileApiParser::parseData(QPromise<std::shared_ptr<FileApiQtcData>> &promise,
                                      const FilePath &replyFilePath,
+                                     const Utils::FilePath &buildDir,
                                      const QString &cmakeBuildType,
                                      QString &errorMessage)
 {
@@ -856,7 +870,11 @@ FileApiData FileApiParser::parseData(QPromise<std::shared_ptr<FileApiQtcData>> &
     result.replyFile = readReplyFile(replyFilePath, errorMessage);
     if (cancelCheck())
         return {};
-    result.cache = readCacheFile(result.replyFile.jsonFile("cache", replyDir), errorMessage);
+    const FilePath cachePathFromReply = result.replyFile.jsonFile("cache", replyDir);
+    if (cachePathFromReply.isEmpty())
+        result.cache = CMakeConfig::fromFile(buildDir / Constants::CMAKE_CACHE_TXT, &errorMessage);
+    else
+        result.cache = readCacheFile(cachePathFromReply, errorMessage);
     if (cancelCheck())
         return {};
     result.cmakeFiles = readCMakeFilesFile(result.replyFile.jsonFile("cmakeFiles", replyDir),
@@ -867,19 +885,21 @@ FileApiData FileApiParser::parseData(QPromise<std::shared_ptr<FileApiQtcData>> &
                                          errorMessage);
 
     if (codeModels.size() == 0) {
-        errorMessage = Tr::tr("CMake project configuration failed. No CMake configuration for "
-                              "build type \"%1\" found.")
+        //: General Messages refers to the output view
+        errorMessage = Tr::tr(
+                           "CMake project configuration failed. No CMake configuration for "
+                           "build type \"%1\" found. Check General Messages for more information.")
                            .arg(cmakeBuildType);
         return result;
     }
 
-    auto it = std::find_if(codeModels.cbegin(), codeModels.cend(),
-                           [cmakeBuildType](const Configuration& cfg) {
+    auto it = std::find_if(codeModels.begin(), codeModels.end(),
+                           [cmakeBuildType](const ConfigurationInfo& cfg) {
                                return QString::compare(cfg.name, cmakeBuildType, Qt::CaseInsensitive) == 0;
                            });
-    if (it == codeModels.cend()) {
+    if (it == codeModels.end()) {
         QStringList buildTypes;
-        for (const Configuration &cfg: codeModels)
+        for (const ConfigurationInfo &cfg: codeModels)
             buildTypes << cfg.name;
 
         if (result.replyFile.isMultiConfig) {
@@ -905,7 +925,7 @@ FileApiData FileApiParser::parseData(QPromise<std::shared_ptr<FileApiQtcData>> &
         if (cancelCheck())
             return {};
         QString targetErrorMessage;
-        TargetDetails td = readTargetFile((replyDir / targetFile).absoluteFilePath(), targetErrorMessage);
+        TargetDetails td = readTargetFile(replyDir, replyDir / targetFile, targetErrorMessage);
         if (targetErrorMessage.isEmpty()) {
             result.targetDetails.emplace_back(std::move(td));
         } else {
@@ -931,9 +951,11 @@ FilePath FileApiParser::scanForCMakeReplyFile(const FilePath &buildDirectory)
 FilePaths FileApiParser::cmakeQueryFilePaths(const FilePath &buildDirectory)
 {
     const FilePath queryDir = buildDirectory / CMAKE_RELATIVE_QUERY_PATH;
-    return transform(CMAKE_QUERY_FILENAMES, [&queryDir](const QString &name) {
-        return queryDir.resolvePath(name);
-    });
+    return {
+        queryDir / "cache-v2",
+        queryDir / "codemodel-v2",
+        queryDir / "cmakeFiles-v1"
+    };
 }
 
 } // CMakeProjectManager::Internal
