@@ -1,35 +1,14 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
+#include <utils/infobar.h>
+
 #include <QIcon>
 #include <QWidget>
 
 #include <QPropertyAnimation>
-#include <QTimer>
 
 QT_BEGIN_NAMESPACE
 class QPainter;
@@ -47,8 +26,9 @@ class FancyTab : public QObject
     Q_PROPERTY(qreal fader READ fader WRITE setFader)
 
 public:
-    FancyTab(QWidget *tabbar)
-        : m_tabbar(tabbar)
+    FancyTab(QWidget *parentTabBar)
+        : QObject(parentTabBar)
+        , m_tabbar(parentTabBar)
     {
         m_animator.setPropertyName("fader");
         m_animator.setTargetObject(this);
@@ -64,6 +44,7 @@ public:
     QString text;
     QString toolTip;
     bool enabled = false;
+    bool visible = true;
     bool hasMenu = false;
 
 private:
@@ -82,10 +63,10 @@ public:
     bool event(QEvent *event) override;
 
     void paintEvent(QPaintEvent *event) override;
-    void paintTab(QPainter *painter, int tabIndex) const;
+    void paintTab(QPainter *painter, int tabIndex, int visibleIndex, QIcon::State iconState) const;
     void mousePressEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
-    void enterEvent(QEvent *event) override;
+    void enterEvent(QEnterEvent *event) override;
     void leaveEvent(QEvent *event) override;
     bool validIndex(int index) const { return index >= 0 && index < m_tabs.count(); }
 
@@ -95,6 +76,8 @@ public:
     void setTabEnabled(int index, bool enable);
     bool isTabEnabled(int index) const;
 
+    void setTabVisible(int index, bool visible);
+
     void insertTab(int index, const QIcon &icon, const QString &label, bool hasMenu)
     {
         auto tab = new FancyTab(this);
@@ -122,7 +105,9 @@ public:
     void setIconsOnly(bool iconOnly);
 
     int count() const { return m_tabs.count(); }
-    QRect tabRect(int index) const;
+    QRect tabRect(int visibleIndex) const;
+
+    int visibleIndex(int index) const;
 
 signals:
     void currentAboutToChange(int index);
@@ -152,14 +137,17 @@ public:
     void insertCornerWidget(int pos, QWidget *widget);
     int cornerWidgetCount() const;
     void setTabToolTip(int index, const QString &toolTip);
+    void setInfoBar(Utils::InfoBar *infoBar);
 
     void paintEvent(QPaintEvent *event) override;
 
     int currentIndex() const;
     QStatusBar *statusBar() const;
+    Utils::InfoBar *infoBar();
 
     void setTabEnabled(int index, bool enable);
     bool isTabEnabled(int index) const;
+    void setTabVisible(int index, bool visible);
 
     void setIconsOnly(bool iconsOnly);
 
@@ -169,7 +157,7 @@ signals:
     void currentAboutToShow(int index);
     void currentChanged(int index);
     void menuTriggered(int index, QMouseEvent *event);
-    void topAreaClicked(Qt::MouseButton button, Qt::KeyboardModifiers modifiers);
+    void topAreaClicked(QMouseEvent *event);
 
 public slots:
     void setCurrentIndex(int index);
@@ -183,6 +171,7 @@ private:
     QStackedLayout *m_modesStack;
     QWidget *m_selectionWidget;
     QStatusBar *m_statusBar;
+    Utils::InfoBarDisplay m_infoBarDisplay;
 };
 
 } // namespace Internal
