@@ -3,13 +3,11 @@
 
 #include "followsymbol_switchmethoddecldef_test.h"
 
+#include "clangdsettings.h"
 #include "cppcodemodelsettings.h"
-#include "cppeditorplugin.h"
 #include "cppeditorwidget.h"
-#include "cppelementevaluator.h"
 #include "cppfollowsymbolundercursor.h"
 #include "cppmodelmanager.h"
-#include "cpptoolsreuse.h"
 #include "cpptoolstestcase.h"
 #include "cppvirtualfunctionassistprovider.h"
 #include "cppvirtualfunctionproposalitem.h"
@@ -32,7 +30,7 @@
 
 #include <QDebug>
 #include <QDir>
-#include <QtTest>
+#include <QTest>
 
 //
 // The following "non-latin1" code points are used in the tests:
@@ -155,7 +153,7 @@ public:
                 = dynamic_cast<VirtualFunctionProposalItem *>(model->proposalItem(i));
 
             const QString text = model->text(i);
-            const int line = item->link().targetLine;
+            const int line = item->link().target.line;
 //            Uncomment for updating/generating reference data:
 //            qDebug("<< OverrideItem(QLatin1String(\"%s\"), %d)", qPrintable(text), line);
             result << OverrideItem(text, line);
@@ -174,7 +172,7 @@ private:
 
 QList<TestDocumentPtr> singleDocument(const QByteArray &source)
 {
-    return QList<TestDocumentPtr>() << CppTestDocument::create(source, "file.cpp");
+    return {CppTestDocument::create(source, "file.cpp")};
 }
 
 /**
@@ -235,14 +233,12 @@ F2TestCase::F2TestCase(CppEditorAction action,
     const QString tag = QLatin1String(QTest::currentDataTag());
     const bool useClangd = m_testKit;
     if (useClangd) {
-        if (curTestName == "testFollowSymbolQObjectConnect"
-                || curTestName == "testFollowSymbolQObjectOldStyleConnect") {
-            QSKIP("TODO: Implement fall-back");
+        if (tag.contains("before keyword") || tag.contains("in keyword")
+            || tag.contains("before parenthesis")) {
+            QSKIP("clangd correctly goes to definition of SIGNAL macro");
         }
         if (curTestName == "testFollowClassOperator" && tag == "backward")
             QSKIP("clangd goes to operator name first");
-        if (tag.toLower().contains("fuzzy"))
-            QSKIP("fuzzy matching is not supposed to work with clangd"); // TODO: Implement fallback as we do with libclang
         if (tag == "baseClassFunctionIntroducedByUsingDeclaration")
             QSKIP("clangd points to the using declaration");
         if (curTestName == "testFollowClassOperatorInOp")
@@ -252,13 +248,13 @@ F2TestCase::F2TestCase(CppEditorAction action,
     // Write files to disk
     CppEditor::Tests::TemporaryDir temporaryDir;
     QVERIFY(temporaryDir.isValid());
-    QString projectFileContent = "CppApplication { files: [";
+    QString projectFileContent = "QtApplication { files: [";
    for (TestDocumentPtr testFile : testFiles) {
         QVERIFY(testFile->baseDirectory().isEmpty());
         testFile->setBaseDirectory(temporaryDir.path());
         QVERIFY(testFile->writeToDisk());
         projectFileContent += QString::fromLatin1("\"%1\",")
-                .arg(testFile->filePath().toString());
+                .arg(testFile->filePath().toFSPathString());
     }
     projectFileContent += "]}\n";
 
@@ -314,7 +310,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
 
         // Rehighlight
         if (!useClangd)
-            waitForRehighlightedSemanticDocument(testFile->m_editorWidget);
+            QVERIFY(waitForRehighlightedSemanticDocument(testFile->m_editorWidget));
     }
 
     // Activate editor of initial test file
@@ -331,9 +327,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
     switch (action) {
     case FollowSymbolUnderCursorAction: {
         CppEditorWidget *widget = initialTestFile->m_editorWidget;
-        if (CppModelManager::instance()->isClangCodeModelActive()) {
-            if (curTestName == "testFollowSymbolQTCREATORBUG7903")
-                QSKIP((curTestName + " is not supported by Clang FollowSymbol").toLatin1());
+        if (useClangd) {
             widget->enableTestMode();
             widget->openLinkUnderCursor();
             break;
@@ -361,7 +355,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
         if (useClangd && tag.endsWith("Var"))
             initialTestFile->m_editorWidget->openLinkUnderCursor();
         else
-            CppEditorPlugin::instance()->switchDeclarationDefinition();
+            initialTestFile->m_editorWidget->switchDeclarationDefinition(/*inNextSplit*/false);
         break;
     default:
         QFAIL("Unknown test action");
@@ -369,8 +363,6 @@ F2TestCase::F2TestCase(CppEditorAction action,
     }
 
     if (useClangd) {
-        QEXPECT_FAIL("infiniteLoopLocalTypedef_QTCREATORBUG-11999",
-                     "clangd bug: Go to definition does not return", Abort);
         if (expectedVirtualFunctionProposal.size() <= 1) {
             QVERIFY(CppEditor::Tests::waitForSignalOrTimeout(EditorManager::instance(),
                                                              &EditorManager::linkOpened, 10000));
@@ -381,7 +373,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
             QObject::connect(&t, &QTimer::timeout, &l, &QEventLoop::quit);
             const IAssistProposal *immediateProposal = nullptr;
             const IAssistProposal *finalProposal = nullptr;
-            QObject::connect(initialTestFile->m_editorWidget, &CppEditorWidget::proposalsReady,
+            QObject::connect(initialTestFile->m_editorWidget, &CppEditorWidget::proposalsReady, &l,
                              [&](const IAssistProposal *i, const IAssistProposal *f) {
                 immediateProposal = i;
                 finalProposal = f;
@@ -405,6 +397,12 @@ F2TestCase::F2TestCase(CppEditorAction action,
     BaseTextEditor *currentTextEditor = dynamic_cast<BaseTextEditor*>(currentEditor);
     QVERIFY(currentTextEditor);
 
+    if (useClangd) {
+        QEXPECT_FAIL("matchFunctionSignatureFuzzy1Forward", "clangd returns decl loc", Abort);
+        QEXPECT_FAIL("matchFunctionSignatureFuzzy2Forward", "clangd returns decl loc", Abort);
+        QEXPECT_FAIL("matchFunctionSignatureFuzzy1Backward", "clangd returns def loc", Abort);
+        QEXPECT_FAIL("matchFunctionSignatureFuzzy2Backward", "clangd returns def loc", Abort);
+    }
     QCOMPARE(currentTextEditor->document()->filePath(), targetTestFile->filePath());
     int expectedLine, expectedColumn;
     if (useClangd && expectedVirtualFunctionProposal.size() == 1) {
@@ -414,6 +412,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
     } else {
         currentTextEditor->convertPosition(targetTestFile->m_targetCursorPosition,
                                            &expectedLine, &expectedColumn);
+        ++expectedColumn;
         if (useClangd && (tag == "classDestructor" || tag == "fromDestructorDefinitionSymbol"
                 || tag == "fromDestructorBody")) {
             --expectedColumn; // clangd goes before the ~, built-in code model after
@@ -422,9 +421,18 @@ F2TestCase::F2TestCase(CppEditorAction action,
 //    qDebug() << "Expected line:" << expectedLine;
 //    qDebug() << "Expected column:" << expectedColumn;
 
-    if (!useClangd) {
+    if (useClangd) {
+        QEXPECT_FAIL("matchFunctionSignature_Follow_8_fuzzy",
+                     "clangd points to declaration", Abort);
+        QEXPECT_FAIL("matchFunctionSignature_Follow_9_fuzzy",
+                     "clangd points to declaration", Abort);
+    } else {
         QEXPECT_FAIL("globalVarFromEnum", "Contributor works on a fix.", Abort);
         QEXPECT_FAIL("matchFunctionSignature_Follow_5", "foo(int) resolved as CallAST", Abort);
+        if (tag.contains("SLOT") && tag.contains("no 2nd QObject"))
+            QEXPECT_FAIL("", "FIXME", Abort);
+        QEXPECT_FAIL(
+            "baseClassViaDecltype", "we cannot properly evaluate decltype at bind time", Abort);
     }
 
     QCOMPARE(currentTextEditor->currentLine(), expectedLine);
@@ -448,7 +456,7 @@ F2TestCase::F2TestCase(CppEditorAction action,
                 first.text = "<base declaration>";
             expectedImmediate << first;
         }
-        expectedImmediate << OverrideItem(QLatin1String("collecting overrides ..."));
+        expectedImmediate << OverrideItem(QLatin1String("collecting overrides..."));
     }
     QCOMPARE(immediateVirtualSymbolResults, expectedImmediate);
     if (useClangd) {
@@ -495,8 +503,11 @@ void FollowSymbolTest::initTestCase()
         return;
 
     // Find suitable kit.
+    // Qt is not actually required for the tests, but we need it for consistency with
+    // configureAsExampleProject().
+    // FIXME: Make configureAsExampleProject() work with non-Qt kits.
     F2TestCase::m_testKit = Utils::findOr(KitManager::kits(), nullptr, [](const Kit *k) {
-        return k->isValid();
+        return k->isValid() && !k->hasWarning() && k->value("QtSupport.QtInformation").isValid();
     });
     if (!F2TestCase::m_testKit)
         QSKIP("This test requires at least one kit to be present");
@@ -792,6 +803,43 @@ void FollowSymbolTest::testSwitchMethodDeclDef_data()
                  "};\n")
             << _("#include \"file.h\"\n"
                  "Foo::@operator int() const { return {}; }\n");
+
+    QTest::newRow("defaultedConstructorDecl2Def")
+        << _("struct Foo {\n"
+             "    @Foo();\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo::$Foo() = default;\n");
+    QTest::newRow("defaultedConstructorDef2Decl")
+        << _("struct Foo {\n"
+             "    $Foo();\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo::Foo() = @default;\n");
+    QTest::newRow("defaultedDestructorDecl2Def")
+        << _("struct Foo {\n"
+             "    ~@Foo();\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo::~$Foo() = default;\n");
+    QTest::newRow("defaultedDestructorDef2Decl")
+        << _("struct Foo {\n"
+             "    ~$Foo();\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo::~Foo() @= default;\n");
+    QTest::newRow("defaultedOperatorDecl2Def")
+        << _("struct Foo {\n"
+             "    Foo& @operator=(const Foo &);\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo& Foo::$operator=(const Foo &) = default;\n");
+    QTest::newRow("defaultedOperatorDef2Decl")
+        << _("struct Foo {\n"
+             "    Foo& $operator=(const Foo &);\n"
+             "};\n")
+        << _("#include \"file.h\"\n"
+             "Foo& Foo::op@erator=(const Foo &) = default;\n");
 }
 
 void FollowSymbolTest::testSwitchMethodDeclDef()
@@ -1249,6 +1297,43 @@ void FollowSymbolTest::testFollowSymbol_data()
         "struct Child : public Parent { void $disconnect(); };\n"
         "void test() { Child c; c.@disconnect(); }\n"
     );
+
+    QTest::newRow("defaultedConstructorDecl2Def")
+        << _("struct Foo {\n"
+             "    @Foo();\n"
+             "};\n"
+             "Foo::$Foo() = default;\n");
+    QTest::newRow("defaultedConstructorDef2Decl")
+        << _("struct Foo {\n"
+             "    $Foo();\n"
+             "};\n"
+             "Foo::@Foo() = default;\n");
+    QTest::newRow("defaultedDestructorDecl2Def")
+        << _("struct Foo {\n"
+             "    ~@Foo();\n"
+             "};\n"
+             "Foo::~$Foo() = default;\n");
+    QTest::newRow("defaultedDestructorDef2Decl")
+        << _("struct Foo {\n"
+             "    ~$Foo();\n"
+             "};\n"
+             "Foo::~@Foo() = default;\n");
+    QTest::newRow("defaultedOperatorDecl2Def")
+        << _("struct Foo {\n"
+             "    Foo& @operator=(const Foo &);\n"
+             "};\n"
+             "Foo& Foo::$operator=(const Foo &) = default;\n");
+    QTest::newRow("defaultedOperatorDef2Decl")
+        << _("struct Foo {\n"
+             "    Foo& $operator=(const Foo &);\n"
+             "};\n"
+             "Foo& Foo::op@erator=(const Foo &) = default;\n");
+    QTest::newRow("baseClassViaDecltype")
+        << _("struct Foo { static const int $_foo = 0; };\n"
+             "struct Bar : public decltype(Foo()) { static const int _bar = @_foo; };\n");
+    QTest::newRow("concept")
+        << _("namespace N { template<typename T> concept $C1 = true; }\n"
+             "static void func(N::@C1 auto p);\n");
 }
 
 void FollowSymbolTest::testFollowSymbol()
@@ -1581,11 +1666,6 @@ void FollowSymbolTest::testFollowSymbolQObjectConnect()
     if (!secondQObjectParam)
         source.replace(" &foo, ", QByteArray());
 
-    if (start >= '7' && !secondQObjectParam) {
-        qWarning("SLOT jump triggers QTCREATORBUG-10265. Skipping.");
-        return;
-    }
-
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
