@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "headerpathfilter.h"
 
@@ -54,9 +32,10 @@ void HeaderPathFilter::process()
         tweakHeaderPaths();
 }
 
-bool HeaderPathFilter::isProjectHeaderPath(const QString &path) const
+bool HeaderPathFilter::isProjectHeaderPath(const FilePath &path) const
 {
-    return path.startsWith(projectDirectory) || path.startsWith(buildDirectory);
+    return path == projectDirectory || path.isChildOf(projectDirectory) || projectDirectory.isEmpty()
+        || path == buildDirectory || path.isChildOf(buildDirectory) || buildDirectory.isEmpty();
 }
 
 void HeaderPathFilter::removeGccInternalIncludePaths()
@@ -66,14 +45,13 @@ void HeaderPathFilter::removeGccInternalIncludePaths()
         return;
     }
 
-    if (projectPart.toolChainInstallDir.isEmpty())
+    if (projectPart.toolchainInstallDir.isEmpty())
         return;
 
-    const Utils::FilePath gccInstallDir = projectPart.toolChainInstallDir;
+    const FilePath gccInstallDir = projectPart.toolchainInstallDir;
     auto isGccInternalInclude = [gccInstallDir](const HeaderPath &headerPath) {
-        const auto filePath = Utils::FilePath::fromString(headerPath.path);
-        return filePath == gccInstallDir.pathAppended("include")
-               || filePath == gccInstallDir.pathAppended("include-fixed");
+        return headerPath.path == gccInstallDir.pathAppended("include")
+               || headerPath.path == gccInstallDir.pathAppended("include-fixed");
     };
 
     Utils::erase(builtInHeaderPaths, isGccInternalInclude);
@@ -115,7 +93,7 @@ HeaderPaths::iterator resourceIterator(HeaderPaths &headerPaths)
     return std::stable_partition(headerPaths.begin(),
                                  headerPaths.end(),
                                  [&](const HeaderPath &headerPath) {
-                                     return includeRegExp.match(headerPath.path).hasMatch();
+                                     return includeRegExp.match(headerPath.path.path()).hasMatch();
                                  });
 }
 
@@ -126,8 +104,8 @@ bool isClangSystemHeaderPath(const HeaderPath &headerPath)
     // For example GCC on macOS uses system clang include path which makes clang code model
     // include incorrect system headers.
     static const QRegularExpression clangIncludeDir(
-        R"(\A.*/lib\d*/clang/\d+\.\d+(\.\d+)?/include\z)");
-    return clangIncludeDir.match(headerPath.path).hasMatch();
+        R"(\A.*/lib\d*/clang/\d+(\.\d+){0,2}/include\z)");
+    return clangIncludeDir.match(headerPath.path.path()).hasMatch();
 }
 
 void removeClangSystemHeaderPaths(HeaderPaths &headerPaths)
@@ -152,19 +130,9 @@ void HeaderPathFilter::tweakHeaderPaths()
 void HeaderPathFilter::addPreIncludesPath()
 {
     if (!projectDirectory.isEmpty()) {
-        const Utils::FilePath rootProjectDirectory = Utils::FilePath::fromString(projectDirectory)
-                .pathAppended(".pre_includes");
-        systemHeaderPaths.push_back(ProjectExplorer::HeaderPath::makeSystem(rootProjectDirectory));
+        const FilePath rootProjectDirectory = projectDirectory / ".pre_includes";
+        systemHeaderPaths.push_back(HeaderPath::makeSystem(rootProjectDirectory));
     }
 }
 
-QString HeaderPathFilter::ensurePathWithSlashEnding(const QString &path)
-{
-    QString pathWithSlashEnding = path;
-    if (!pathWithSlashEnding.isEmpty() && *pathWithSlashEnding.rbegin() != '/')
-        pathWithSlashEnding.push_back('/');
-
-    return pathWithSlashEnding;
-}
-
 } // namespace CppEditor::Internal
