@@ -22,7 +22,6 @@
 #include <projectexplorer/headerpath.h>
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <projectexplorer/projectexplorertr.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projectnodes.h>
@@ -41,6 +40,7 @@
 #include <utils/fileutils.h>
 #include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QDir>
 #include <QFileInfo>
@@ -91,14 +91,13 @@ public:
         const Utils::FilePairs &filesToRename,
         Utils::FilePaths *notRenamed) final;
     bool addFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
-    QString name() const final { return QLatin1String("generic"); }
 
-    FilePath filesFilePath() const { return ::FilePath::fromString(m_filesFileName); }
+    FilePath filesFilePath() const { return m_filesFilePath; }
 
     void refresh(RefreshOptions options);
 
     bool saveRawFileList(const QStringList &rawFileList);
-    bool saveRawList(const QStringList &rawList, const QString &fileName);
+    bool saveRawList(const QStringList &rawList, const FilePath &filePath);
     void parse(RefreshOptions options);
 
     using SourceFile = QPair<FilePath, QStringList>;
@@ -114,11 +113,11 @@ public:
     void removeFiles(const FilePaths &filesToRemove);
 
 private:
-    QString m_filesFileName;
-    QString m_includesFileName;
-    QString m_configFileName;
-    QString m_cxxflagsFileName;
-    QString m_cflagsFileName;
+    FilePath m_filesFilePath;
+    FilePath m_includesFilePath;
+    FilePath m_configFilePath;
+    FilePath m_cxxflagsFilePath;
+    FilePath m_cflagsFilePath;
     QStringList m_rawFileList;
     SourceFiles m_files;
     QHash<QString, QString> m_rawListEntries;
@@ -140,7 +139,7 @@ class GenericBuildConfiguration final : public BuildConfiguration
 {
 public:
     GenericBuildConfiguration(Target *target, Id id)
-        : BuildConfiguration(target, id), m_buildSystem(new GenericBuildSystem(this))
+        : BuildConfiguration(target, id)
     {
         setConfigWidgetDisplayName(GenericProjectManager::Tr::tr("Generic Manager"));
         setBuildDirectoryHistoryCompleter("Generic.BuildDir.History");
@@ -154,17 +153,11 @@ public:
         updateCacheAndEmitEnvironmentChanged();
     }
 
-    ~GenericBuildConfiguration() { delete m_buildSystem; }
-
 private:
     void addToEnvironment(Environment &env) const final
     {
         QtSupport::QtKitAspect::addHostBinariesToPath(kit(), env);
     }
-
-    BuildSystem *buildSystem() const { return m_buildSystem; }
-
-    GenericBuildSystem * const m_buildSystem;
 };
 
 class GenericBuildConfigurationFactory final : public BuildConfigurationFactory
@@ -180,12 +173,12 @@ public:
 
         setBuildGenerator([](const Kit *, const FilePath &projectPath, bool forSetup) {
             BuildInfo info;
-            info.typeName = ProjectExplorer::Tr::tr("Build");
+            info.typeName = msgBuildConfigurationBuild();
             info.buildDirectory = forSetup ? projectPath.absolutePath() : projectPath;
 
             if (forSetup)  {
                 //: The name of the build configuration created by default for a generic project.
-                info.displayName = ProjectExplorer::Tr::tr("Default");
+                info.displayName = msgBuildConfigurationDefault();
             }
 
             return QList<BuildInfo>{info};
@@ -198,10 +191,9 @@ public:
 // GenericProject
 //
 
-static bool writeFile(const QString &filePath, const QString &contents)
+static bool writeFile(const FilePath &filePath, const QString &contents)
 {
-    FileSaver saver(FilePath::fromString(filePath),
-                    QIODevice::Text | QIODevice::WriteOnly);
+    FileSaver saver(filePath, QIODevice::Text | QIODevice::WriteOnly);
     return saver.write(contents.toUtf8()) && saver.finalize();
 }
 
@@ -213,9 +205,10 @@ public:
     explicit GenericProject(const FilePath &filePath)
         : Project(Constants::GENERICMIMETYPE, filePath)
     {
-        setId(Constants::GENERICPROJECT_ID);
+        setType(Constants::GENERICPROJECT_ID);
         setProjectLanguages(Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
         setDisplayName(filePath.completeBaseName());
+        setBuildSystemCreator<GenericBuildSystem>("generic");
     }
 
     void editFilesTriggered();
@@ -242,40 +235,32 @@ GenericBuildSystem::GenericBuildSystem(BuildConfiguration *bc)
             refresh(Everything);
     });
 
-    const QFileInfo fileInfo = projectFilePath().toFileInfo();
-    const QDir dir = fileInfo.dir();
+    const FilePath dir = projectFilePath().parentDir().absoluteFilePath();
+    const QString projectName = projectFilePath().completeBaseName();
 
-    const QString projectName = fileInfo.completeBaseName();
+    m_filesFilePath = dir.pathAppended(projectName + ".files");
+    m_includesFilePath = dir.pathAppended(projectName + ".includes");
+    m_configFilePath = dir.pathAppended(projectName + ".config");
 
-    m_filesFileName    = QFileInfo(dir, projectName + ".files").absoluteFilePath();
-    m_includesFileName = QFileInfo(dir, projectName + ".includes").absoluteFilePath();
-    m_configFileName = QFileInfo(dir, projectName + ".config").absoluteFilePath();
-
-    const QFileInfo cxxflagsFileInfo(dir, projectName + ".cxxflags");
-    m_cxxflagsFileName = cxxflagsFileInfo.absoluteFilePath();
-    if (!cxxflagsFileInfo.exists()) {
-        QTC_CHECK(writeFile(m_cxxflagsFileName, Constants::GENERICPROJECT_CXXFLAGS_FILE_TEMPLATE));
+    m_cxxflagsFilePath = dir.pathAppended(projectName + ".cxxflags");
+    if (!m_cxxflagsFilePath.exists()) {
+        QTC_CHECK(writeFile(m_cxxflagsFilePath, Constants::GENERICPROJECT_CXXFLAGS_FILE_TEMPLATE));
     }
 
-    const QFileInfo cflagsFileInfo(dir, projectName + ".cflags");
-    m_cflagsFileName = cflagsFileInfo.absoluteFilePath();
-    if (!cflagsFileInfo.exists()) {
-        QTC_CHECK(writeFile(m_cflagsFileName, Constants::GENERICPROJECT_CFLAGS_FILE_TEMPLATE));
+    m_cflagsFilePath = dir.pathAppended(projectName + ".cflags");
+    if (!m_cflagsFilePath.exists()) {
+        QTC_CHECK(writeFile(m_cflagsFilePath, Constants::GENERICPROJECT_CFLAGS_FILE_TEMPLATE));
     }
 
-    project()->setExtraProjectFiles({FilePath::fromString(m_filesFileName),
-                                     FilePath::fromString(m_includesFileName),
-                                     FilePath::fromString(m_configFileName),
-                                     FilePath::fromString(m_cxxflagsFileName),
-                                     FilePath::fromString(m_cflagsFileName)});
+    project()->setExtraProjectFiles({m_filesFilePath,
+                                     m_includesFilePath,
+                                     m_configFilePath,
+                                     m_cxxflagsFilePath,
+                                     m_cflagsFilePath});
 
     connect(&m_deployFileWatcher, &FileSystemWatcher::fileChanged,
             this, &GenericBuildSystem::updateDeploymentData);
-
-    connect(bc->target(), &Target::activeBuildConfigurationChanged, this, [this] {
-        refresh(Everything);
-    });
-    connect(project(), &Project::activeTargetChanged, this, [this] {
+    connect(project(), &Project::activeBuildConfigurationChanged, this, [this] {
         refresh(Everything);
     });
 }
@@ -290,11 +275,11 @@ void GenericBuildSystem::triggerParsing()
     refresh(Everything);
 }
 
-static QStringList readLines(const QString &absoluteFileName)
+static QStringList readLines(const FilePath &absoluteFileName)
 {
     QStringList lines;
 
-    QFile file(absoluteFileName);
+    QFile file(absoluteFileName.toFSPathString());
     if (file.open(QFile::ReadOnly)) {
         QTextStream stream(&file);
 
@@ -312,14 +297,13 @@ static QStringList readLines(const QString &absoluteFileName)
 
 bool GenericBuildSystem::saveRawFileList(const QStringList &rawFileList)
 {
-    bool result = saveRawList(rawFileList, m_filesFileName);
+    bool result = saveRawList(rawFileList, m_filesFilePath);
     refresh(Files);
     return result;
 }
 
-bool GenericBuildSystem::saveRawList(const QStringList &rawList, const QString &fileName)
+bool GenericBuildSystem::saveRawList(const QStringList &rawList, const FilePath &filePath)
 {
-    const FilePath filePath = FilePath::fromString(fileName);
     FileChangeBlocker changeGuard(filePath);
     // Make sure we can open the file for writing
     FileSaver saver(filePath, QIODevice::Text);
@@ -329,17 +313,10 @@ bool GenericBuildSystem::saveRawList(const QStringList &rawList, const QString &
             stream << filePath << '\n';
         saver.setResult(&stream);
     }
-    bool result = saver.finalize(ICore::dialogParent());
-    return result;
-}
-
-static void insertSorted(QStringList *list, const QString &value)
-{
-    const auto it = std::lower_bound(list->begin(), list->end(), value);
-    if (it == list->end())
-        list->append(value);
-    else if (*it > value)
-        list->insert(it, value);
+    const Result<> result = saver.finalize();
+    if (!result)
+        FileUtils::showError(result.error());
+    return result.has_value();
 }
 
 bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths *)
@@ -355,11 +332,11 @@ bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths
         newList.erase(std::unique(newList.begin(), newList.end()), newList.end());
     } else {
         for (const QString &filePath : filePaths)
-            insertSorted(&newList, baseDir.relativeFilePath(filePath));
+            Utils::insertSorted(&newList, baseDir.relativeFilePath(filePath));
     }
 
     const auto includes = transform<QSet<QString>>(m_projectIncludePaths,
-                                                   [](const HeaderPath &hp) { return hp.path; });
+                                                   [](const HeaderPath &hp) { return hp.path.path(); });
     QSet<QString> toAdd;
 
     for (const QString &filePath : filePaths) {
@@ -378,8 +355,8 @@ bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths
         m_rawProjectIncludePaths.append(relative);
     }
 
-    bool result = saveRawList(newList, m_filesFileName);
-    result &= saveRawList(m_rawProjectIncludePaths, m_includesFileName);
+    bool result = saveRawList(newList, m_filesFilePath);
+    result &= saveRawList(m_rawProjectIncludePaths, m_includesFilePath);
     refresh(Everything);
 
     return result;
@@ -436,7 +413,7 @@ bool GenericBuildSystem::renameFiles(Node *, const FilePairs &filesToRename, Fil
 
         QDir baseDir(projectDirectory().toUrlishString());
         newList.removeAt(index);
-        insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toUrlishString()));
+        Utils::insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toUrlishString()));
     }
 
     if (!saveRawFileList(newList)) {
@@ -448,14 +425,14 @@ bool GenericBuildSystem::renameFiles(Node *, const FilePairs &filesToRename, Fil
     return success;
 }
 
-static QStringList readFlags(const QString &filePath)
+static QStringList readFlags(const FilePath &filePath)
 {
     const QStringList lines = readLines(filePath);
     if (lines.isEmpty())
         return {};
     QStringList flags;
     for (const auto &line : lines)
-        flags.append(ProcessArgs::splitArgs(line, HostOsInfo::hostOs()));
+        flags.append(ProcessArgs::splitArgs(line, filePath.osType()));
     return flags;
 }
 
@@ -463,15 +440,15 @@ void GenericBuildSystem::parse(RefreshOptions options)
 {
     if (options & Files) {
         m_rawListEntries.clear();
-        m_rawFileList = readLines(m_filesFileName);
+        m_rawFileList = readLines(m_filesFilePath);
         m_files = processEntries(m_rawFileList, &m_rawListEntries);
     }
 
     if (options & Configuration) {
-        m_rawProjectIncludePaths = readLines(m_includesFileName);
+        m_rawProjectIncludePaths = readLines(m_includesFilePath);
         QStringList normalPaths;
         QStringList frameworkPaths;
-        const auto baseDir = FilePath::fromString(m_includesFileName).parentDir();
+        const FilePath baseDir = m_includesFilePath.parentDir();
         for (const QString &rawPath : std::as_const(m_rawProjectIncludePaths)) {
             if (rawPath.startsWith("-F"))
                 frameworkPaths << rawPath.mid(2);
@@ -485,15 +462,15 @@ void GenericBuildSystem::parse(RefreshOptions options)
         };
         m_projectIncludePaths = toUserHeaderPaths(expandedPaths(normalPaths));
         m_projectIncludePaths << toFrameworkHeaderPaths(expandedPaths(frameworkPaths));
-        m_cxxflags = readFlags(m_cxxflagsFileName);
-        m_cflags = readFlags(m_cflagsFileName);
+        m_cxxflags = readFlags(m_cxxflagsFilePath);
+        m_cflags = readFlags(m_cflagsFilePath);
     }
 }
 
 FilePath GenericBuildSystem::findCommonSourceRoot()
 {
     if (m_files.isEmpty())
-        return FilePath::fromFileInfo(QFileInfo(m_filesFileName));
+        return m_filesFilePath;
 
     QString root = m_files.front().first.toUrlishString();
     for (const SourceFile &sourceFile : std::as_const(m_files)) {
@@ -537,16 +514,11 @@ void GenericBuildSystem::refresh(RefreshOptions options)
         }
         newRoot->addNestedNodes(std::move(fileNodes), baseDir);
 
-        newRoot->addNestedNode(std::make_unique<FileNode>(FilePath::fromString(m_filesFileName),
-                                                          FileType::Project));
-        newRoot->addNestedNode(std::make_unique<FileNode>(FilePath::fromString(m_includesFileName),
-                                                          FileType::Project));
-        newRoot->addNestedNode(std::make_unique<FileNode>(FilePath::fromString(m_configFileName),
-                                                          FileType::Project));
-        newRoot->addNestedNode(std::make_unique<FileNode>(FilePath::fromString(m_cxxflagsFileName),
-                                                          FileType::Project));
-        newRoot->addNestedNode(std::make_unique<FileNode>(FilePath::fromString(m_cflagsFileName),
-                                                          FileType::Project));
+        newRoot->addNestedNode(std::make_unique<FileNode>(m_filesFilePath, FileType::Project));
+        newRoot->addNestedNode(std::make_unique<FileNode>(m_includesFilePath, FileType::Project));
+        newRoot->addNestedNode(std::make_unique<FileNode>(m_configFilePath, FileType::Project));
+        newRoot->addNestedNode(std::make_unique<FileNode>(m_cxxflagsFilePath, FileType::Project));
+        newRoot->addNestedNode(std::make_unique<FileNode>(m_cflagsFilePath, FileType::Project));
 
         newRoot->compress();
         setRootProjectNode(std::move(newRoot));
@@ -569,13 +541,8 @@ void GenericBuildSystem::refresh(RefreshOptions options)
 GenericBuildSystem::SourceFiles GenericBuildSystem::processEntries(
         const QStringList &paths, QHash<QString, QString> *map) const
 {
-    const BuildConfiguration *const buildConfig = target()->activeBuildConfiguration();
-
-    const Environment buildEnv = buildConfig ? buildConfig->environment()
-                                             : Environment::systemEnvironment();
-
-    const MacroExpander *expander = buildConfig ? buildConfig->macroExpander()
-                                                : target()->macroExpander();
+    const Environment buildEnv = buildConfiguration()->environment();
+    const MacroExpander *expander = buildConfiguration()->macroExpander();
 
     const QDir projectDir(projectDirectory().toUrlishString());
 
@@ -618,8 +585,6 @@ void GenericBuildSystem::refreshCppCodeModel()
 {
     if (!m_cppCodeModelUpdater)
         return;
-    if (target() != project()->activeTarget())
-        return;
     QtSupport::CppKitInfo kitInfo(kit());
     QTC_ASSERT(kitInfo.isValid(), return);
 
@@ -628,17 +593,15 @@ void GenericBuildSystem::refreshCppCodeModel()
     rpp.setProjectFileLocation(projectFilePath());
     rpp.setQtVersion(kitInfo.projectPartQtVersion);
     rpp.setHeaderPaths(m_projectIncludePaths);
-    rpp.setConfigFileName(m_configFileName);
+    rpp.setConfigFilePath(m_configFilePath);
     rpp.setFlagsForCxx({nullptr, m_cxxflags, projectDirectory()});
     rpp.setFlagsForC({nullptr, m_cflags, projectDirectory()});
 
-    static const auto sourceFilesToStringList = [](const SourceFiles &sourceFiles) {
-        return Utils::transform(sourceFiles, [](const SourceFile &f) {
-            return f.first.toUrlishString();
-        });
+    static const auto sourcePaths = [](const SourceFiles &sourceFiles) {
+        return Utils::transform(sourceFiles, &SourceFile::first);
     };
-    rpp.setFiles(sourceFilesToStringList(m_files));
-    rpp.setPreCompiledHeaders(sourceFilesToStringList(
+    rpp.setFiles(sourcePaths(m_files));
+    rpp.setPreCompiledHeaders(sourcePaths(
         Utils::filtered(m_files, [](const SourceFile &f) { return f.second.contains("pch"); })));
 
     m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), {rpp}});
@@ -648,9 +611,7 @@ void GenericBuildSystem::updateDeploymentData()
 {
     static const QString fileName("QtCreatorDeployment.txt");
     FilePath deploymentFilePath;
-    BuildConfiguration *bc = target()->activeBuildConfiguration();
-    if (bc)
-        deploymentFilePath = bc->buildDirectory().pathAppended(fileName);
+    deploymentFilePath = buildConfiguration()->buildDirectory().pathAppended(fileName);
 
     bool hasDeploymentData = deploymentFilePath.exists();
     if (!hasDeploymentData) {
@@ -661,7 +622,7 @@ void GenericBuildSystem::updateDeploymentData()
         DeploymentData deploymentData;
         deploymentData.addFilesFromDeploymentFile(deploymentFilePath, projectDirectory());
         setDeploymentData(deploymentData);
-        if (m_deployFileWatcher.filePaths() != FilePaths{deploymentFilePath}) {
+        if (m_deployFileWatcher.files() != FilePaths{deploymentFilePath}) {
             m_deployFileWatcher.clear();
             m_deployFileWatcher.addFile(deploymentFilePath,
                                         FileSystemWatcher::WatchModifiedDate);
@@ -672,9 +633,8 @@ void GenericBuildSystem::updateDeploymentData()
 void GenericBuildSystem::removeFiles(const FilePaths &filesToRemove)
 {
     if (removeFiles(nullptr, filesToRemove, nullptr) == RemovedFilesFromProject::Error) {
-        TaskHub::addTask(BuildSystemTask(Task::Error,
-                                         Tr::tr("Project files list update failed."),
-                                         filesFilePath()));
+        TaskHub::addTask<BuildSystemTask>(
+            Task::Error, Tr::tr("Project files list update failed."), filesFilePath());
     }
 }
 
@@ -697,8 +657,12 @@ Project::RestoreResult GenericProject::fromMap(const Store &map, QString *errorM
             removeTarget(t);
             continue;
         }
-        if (!t->activeRunConfiguration())
-            t->addRunConfiguration(new CustomExecutableRunConfiguration(t));
+        for (BuildConfiguration * const bc : t->buildConfigurations()) {
+            if (!bc->activeRunConfiguration()) {
+                bc->addRunConfiguration(
+                    new CustomExecutableRunConfiguration(bc), NameHandling::Uniquify);
+            }
+        }
     }
 
     if (auto bs = activeBuildSystem())
