@@ -23,7 +23,6 @@
 #include <QLabel>
 #include <QPushButton>
 #include <QRegularExpression>
-#include <QUuid>
 
 using namespace Utils;
 
@@ -69,19 +68,21 @@ GitLabServerWidget::GitLabServerWidget(Mode m, QWidget *parent)
     , m_mode(m)
 {
     m_host.setLabelText(Tr::tr("Host:"));
-    m_host.setDisplayStyle(m == Display ? Utils::StringAspect::LabelDisplay
-                                        : Utils::StringAspect::LineEditDisplay);
-    m_host.setValidationFunction([](Utils::FancyLineEdit *l, QString *) {
-        return hostValid(l->text());
+    m_host.setDisplayStyle(m == Display ? StringAspect::LabelDisplay
+                                        : StringAspect::LineEditDisplay);
+    m_host.setValidationFunction([](const QString &text) -> Result<> {
+        if (hostValid(text))
+            return ResultOk;
+        return ResultError(QString());
     });
 
     m_description.setLabelText(Tr::tr("Description:"));
-    m_description.setDisplayStyle(m == Display ? Utils::StringAspect::LabelDisplay
-                                               : Utils::StringAspect::LineEditDisplay);
+    m_description.setDisplayStyle(m == Display ? StringAspect::LabelDisplay
+                                               : StringAspect::LineEditDisplay);
 
     m_token.setLabelText(Tr::tr("Access token:"));
-    m_token.setDisplayStyle(m == Display ? Utils::StringAspect::LabelDisplay
-                                         : Utils::StringAspect::LineEditDisplay);
+    m_token.setDisplayStyle(m == Display ? StringAspect::LabelDisplay
+                                         : StringAspect::LineEditDisplay);
     m_token.setVisible(m == Edit);
 
     m_port.setLabelText(Tr::tr("Port:"));
@@ -89,7 +90,7 @@ GitLabServerWidget::GitLabServerWidget(Mode m, QWidget *parent)
     m_port.setValue(GitLabServer::defaultPort);
     m_port.setEnabled(m == Edit);
     m_secure.setLabelText(Tr::tr("HTTPS:"));
-    m_secure.setLabelPlacement(Utils::BoolAspect::LabelPlacement::InExtraLabel);
+    m_secure.setLabelPlacement(BoolAspect::LabelPlacement::InExtraLabel);
     m_secure.setDefaultValue(true);
     m_secure.setEnabled(m == Edit);
 
@@ -102,7 +103,7 @@ GitLabServerWidget::GitLabServerWidget(Mode m, QWidget *parent)
             m_token, br,
             m_port, br,
             m_secure,
-            m == Edit ? normalMargin : noMargin
+            m == Edit ? &Layout::setNormalMargins : &Layout::setNoMargins
         },
     }.attachTo(this);
 }
@@ -110,12 +111,12 @@ GitLabServerWidget::GitLabServerWidget(Mode m, QWidget *parent)
 GitLabServer GitLabServerWidget::gitLabServer() const
 {
     GitLabServer result;
-    result.id = m_mode == Edit ? Utils::Id::fromName(QUuid::createUuid().toByteArray()) : m_id;
-    result.host = m_host.value();
-    result.description = m_description.value();
-    result.token = m_token.value();
-    result.port = m_port.value();
-    result.secure = m_secure.value();
+    result.id = m_mode == Edit ? Id::generate() : m_id;
+    result.host = m_host();
+    result.description = m_description();
+    result.token = m_token();
+    result.port = m_port();
+    result.secure = m_secure();
     return result;
 }
 
@@ -132,7 +133,7 @@ void GitLabServerWidget::setGitLabServer(const GitLabServer &server)
 class GitLabOptionsWidget : public Core::IOptionsPageWidget
 {
 public:
-    explicit GitLabOptionsWidget(GitLabParameters *parameters);
+    GitLabOptionsWidget();
 
 private:
     void showEditServerDialog();
@@ -151,8 +152,8 @@ private:
     FilePathAspect m_curl;
 };
 
-GitLabOptionsWidget::GitLabOptionsWidget(GitLabParameters *params)
-    : m_parameters(params)
+GitLabOptionsWidget::GitLabOptionsWidget()
+    : m_parameters(&gitLabParameters())
 {
     auto defaultLabel = new QLabel(Tr::tr("Default:"), this);
     m_defaultGitLabServer = new QComboBox(this);
@@ -178,17 +179,18 @@ GitLabOptionsWidget::GitLabOptionsWidget(GitLabParameters *params)
         }, Column { m_add, m_edit, m_remove, st },
     }.attachTo(this);
 
-    m_curl.setFilePath(params->curl);
+    m_curl.setValue(m_parameters->curl);
 
-    for (const auto &gitLabServer : params->gitLabServers) {
+    for (const auto &gitLabServer : std::as_const(m_parameters->gitLabServers)) {
         m_defaultGitLabServer->addItem(gitLabServer.displayString(),
                                        QVariant::fromValue(gitLabServer));
     }
 
-    const GitLabServer found = params->currentDefaultServer();
+    const GitLabServer found = m_parameters->currentDefaultServer();
     if (found.id.isValid()) {
         m_defaultGitLabServer->setCurrentIndex(m_defaultGitLabServer->findData(
                                                    QVariant::fromValue(found)));
+        m_gitLabServerWidget->setGitLabServer(found);
     }
     updateButtonsState();
 
@@ -298,12 +300,12 @@ void GitLabOptionsWidget::updateButtonsState()
 
 // GitLabOptionsPage
 
-GitLabOptionsPage::GitLabOptionsPage(GitLabParameters *p)
+GitLabOptionsPage::GitLabOptionsPage()
 {
     setId(Constants::GITLAB_SETTINGS);
     setDisplayName(Tr::tr("GitLab"));
     setCategory(VcsBase::Constants::VCS_SETTINGS_CATEGORY);
-    setWidgetCreator([p] { return new GitLabOptionsWidget(p); });
+    setWidgetCreator([] { return new GitLabOptionsWidget; });
 }
 
 } // namespace GitLab
