@@ -57,7 +57,6 @@
 #include <QTreeWidget>
 
 constexpr char typeIdKey[] = "typeId";
-constexpr char nameKey[] = "name";
 constexpr char idKey[] = "id";
 constexpr char enabledKey[] = "enabled";
 constexpr char startupBehaviorKey[] = "startupBehavior";
@@ -65,8 +64,6 @@ constexpr char mimeTypeKey[] = "mimeType";
 constexpr char filePatternKey[] = "filePattern";
 constexpr char initializationOptionsKey[] = "initializationOptions";
 constexpr char configurationKey[] = "configuration";
-constexpr char executableKey[] = "executable";
-constexpr char argumentsKey[] = "arguments";
 constexpr char settingsGroupKey[] = "LanguageClient";
 constexpr char clientsKey[] = "clients";
 constexpr char typedClientsKey[] = "typedClients";
@@ -75,6 +72,7 @@ constexpr char mimeType[] = "application/language.client.setting";
 
 using namespace ProjectExplorer;
 using namespace Utils;
+using namespace TextEditor;
 
 namespace LanguageClient {
 
@@ -198,16 +196,16 @@ private:
         QWidget *widget = nullptr;
     } m_currentSettings;
 
-    void addItem(const Utils::Id &clientTypeId);
+    void addItem(const Id &clientTypeId);
     void deleteItem();
 
     FilterProxy m_settings;
     QSet<QString> &m_changedSettings;
 };
 
-QMap<Utils::Id, ClientType> &clientTypes()
+QMap<Id, ClientType> &clientTypes()
 {
-    static QMap<Utils::Id, ClientType> types;
+    static QMap<Id, ClientType> types;
     return types;
 }
 
@@ -299,7 +297,7 @@ void LanguageClientSettingsPageWidget::applyCurrentSettings()
     }
 }
 
-BaseSettings *generateSettings(const Utils::Id &clientTypeId)
+BaseSettings *generateSettings(const Id &clientTypeId)
 {
     if (auto generator = clientTypes().value(clientTypeId).generator) {
         auto settings = generator();
@@ -309,7 +307,7 @@ BaseSettings *generateSettings(const Utils::Id &clientTypeId)
     return nullptr;
 }
 
-void LanguageClientSettingsPageWidget::addItem(const Utils::Id &clientTypeId)
+void LanguageClientSettingsPageWidget::addItem(const Id &clientTypeId)
 {
     auto newSettings = generateSettings(clientTypeId);
     QTC_ASSERT(newSettings, return);
@@ -402,7 +400,7 @@ QVariant LanguageClientSettingsModel::data(const QModelIndex &index, int role) c
     if (!setting)
         return QVariant();
     if (role == Qt::DisplayRole)
-        return Utils::globalMacroExpander()->expand(setting->m_name);
+        return setting->name();
     else if (role == Qt::CheckStateRole)
         return setting->m_enabled ? Qt::Checked : Qt::Unchecked;
     else if (role == idRole)
@@ -481,7 +479,7 @@ bool LanguageClientSettingsModel::dropMimeData(
         return true;
 
     const QString id = QString::fromUtf8(data->data(mimeType));
-    auto setting = Utils::findOrDefault(m_settings, [id](const BaseSettings *setting) {
+    auto setting = findOrDefault(m_settings, [id](const BaseSettings *setting) {
         return setting->m_id == id;
     });
     if (!setting)
@@ -542,6 +540,16 @@ QModelIndex LanguageClientSettingsModel::indexForSetting(BaseSettings *setting)
     return index < 0 ? QModelIndex() : createIndex(index, 0, setting);
 }
 
+// BaseSettings
+
+BaseSettings::BaseSettings()
+{
+    name.setSettingsKey("name");
+    name.setDefaultValue("New Language Server");
+    name.setLabelText(Tr::tr("Name:"));
+    name.setDisplayStyle(StringAspect::LineEditDisplay);
+}
+
 QJsonObject BaseSettings::initializationOptions() const
 {
     return QJsonDocument::fromJson(Utils::globalMacroExpander()->
@@ -560,12 +568,10 @@ QJsonValue BaseSettings::configuration() const
 
 bool BaseSettings::applyFromSettingsWidget(QWidget *widget)
 {
-    bool changed = false;
+    bool changed = isDirty();
+    AspectContainer::apply();
+
     if (auto settingsWidget = qobject_cast<BaseSettingsWidget *>(widget)) {
-        if (m_name != settingsWidget->name()) {
-            m_name = settingsWidget->name();
-            changed = true;
-        }
         if (m_languageFilter != settingsWidget->filter()) {
             m_languageFilter = settingsWidget->filter();
             changed = true;
@@ -587,9 +593,18 @@ QWidget *BaseSettings::createSettingsWidget(QWidget *parent) const
     return new BaseSettingsWidget(this, parent);
 }
 
+BaseSettings *BaseSettings::copy() const
+{
+    BaseSettings *other = create();
+    Store store;
+    toMap(store);
+    other->fromMap(store);
+    return other;
+}
+
 bool BaseSettings::isValid() const
 {
-    return !m_name.isEmpty();
+    return !name().isEmpty();
 }
 
 bool BaseSettings::isValidOnBuildConfiguration(BuildConfiguration *) const
@@ -626,7 +641,7 @@ Client *BaseSettings::createClient(BuildConfiguration *bc) const
     QTC_ASSERT(client, return nullptr);
 
     if (client->name().isEmpty())
-        client->setName(Utils::globalMacroExpander()->expand(m_name));
+        client->setName(name());
 
     client->setSupportedLanguage(m_languageFilter);
     client->setInitializationOptions(initializationOptions());
@@ -643,8 +658,8 @@ Client *BaseSettings::createClient(BaseClientInterface *interface) const
 
 void BaseSettings::toMap(Store &map) const
 {
+    AspectContainer::toMap(map);
     map.insert(typeIdKey, m_settingsTypeId.toSetting());
-    map.insert(nameKey, m_name);
     map.insert(idKey, m_id);
     map.insert(enabledKey, m_enabled);
     map.insert(startupBehaviorKey, m_startBehavior);
@@ -656,7 +671,7 @@ void BaseSettings::toMap(Store &map) const
 
 void BaseSettings::fromMap(const Store &map)
 {
-    m_name = map[nameKey].toString();
+    AspectContainer::fromMap(map);
     m_id = map.value(idKey, QUuid::createUuid().toString()).toString();
     m_enabled = map[enabledKey].toBool();
     m_startBehavior = BaseSettings::StartBehavior(
@@ -666,6 +681,7 @@ void BaseSettings::fromMap(const Store &map)
     m_languageFilter.filePattern.removeAll(QString()); // remove empty entries
     m_initializationOptions = map[initializationOptionsKey].toString();
     m_configuration = map[configurationKey].toString();
+    m_settingsTypeId = Id::fromSetting(map[typeIdKey]);
 }
 
 static LanguageClientSettingsPage &settingsPage()
@@ -685,9 +701,9 @@ bool LanguageClientSettings::initialized()
     return settingsPage().initialized();
 }
 
-QList<Utils::Store> LanguageClientSettings::storesBySettingsType(Utils::Id settingsTypeId)
+QList<Store> LanguageClientSettings::storesBySettingsType(Id settingsTypeId)
 {
-    QList<Utils::Store> result;
+    QList<Store> result;
 
     QtcSettings *settingsIn = Core::ICore::settings();
     settingsIn->beginGroup(settingsGroupKey);
@@ -766,8 +782,8 @@ void LanguageClientSettings::toSettings(QtcSettings *settings,
             return variantFromStore(store);
         });
     };
-    auto isStdioSetting = Utils::equal(&BaseSettings::m_settingsTypeId,
-                                       Utils::Id(Constants::LANGUAGECLIENT_STDIO_SETTINGS_ID));
+    auto isStdioSetting = Utils::equal(
+        &BaseSettings::m_settingsTypeId, Id(Constants::LANGUAGECLIENT_STDIO_SETTINGS_ID));
     auto [stdioSettings, typedSettings] = Utils::partition(languageClientSettings, isStdioSetting);
     settings->setValue(clientsKey, transform(stdioSettings));
 
@@ -804,23 +820,21 @@ void LanguageClientSettings::setOutlineComboBoxSorted(bool sorted)
     settings->endGroup();
 }
 
-bool StdIOSettings::applyFromSettingsWidget(QWidget *widget)
+// StdIOSettings
+
+StdIOSettings::StdIOSettings()
 {
-    bool changed = false;
-    if (auto settingsWidget = qobject_cast<StdIOSettingsWidget *>(widget)) {
-        changed = BaseSettings::applyFromSettingsWidget(settingsWidget);
-        if (m_executable != settingsWidget->executable()) {
-            m_executable = settingsWidget->executable();
-            changed = true;
-        }
-        if (m_arguments != settingsWidget->arguments()) {
-            m_arguments = settingsWidget->arguments();
-            changed = true;
-        }
-    }
-    return changed;
+    executable.setSettingsKey("executable");
+    executable.setExpectedKind(PathChooser::ExistingCommand);
+    executable.setLabelText(Tr::tr("Executable:"));
+
+    arguments.setSettingsKey("arguments");
+    arguments.setDisplayStyle(StringAspect::LineEditDisplay);
+    arguments.setLabelText(Tr::tr("Arguments:"));
 }
 
+StdIOSettings::~StdIOSettings() = default;
+
 QWidget *StdIOSettings::createSettingsWidget(QWidget *parent) const
 {
     return new StdIOSettingsWidget(this, parent);
@@ -828,31 +842,12 @@ QWidget *StdIOSettings::createSettingsWidget(QWidget *parent) const
 
 bool StdIOSettings::isValid() const
 {
-    return BaseSettings::isValid() && !m_executable.isEmpty();
+    return BaseSettings::isValid() && !executable().isEmpty();
 }
 
-void StdIOSettings::toMap(Store &map) const
+CommandLine StdIOSettings::command() const
 {
-    BaseSettings::toMap(map);
-    map.insert(executableKey, m_executable.toSettings());
-    map.insert(argumentsKey, m_arguments);
-}
-
-void StdIOSettings::fromMap(const Store &map)
-{
-    BaseSettings::fromMap(map);
-    m_executable = Utils::FilePath::fromSettings(map[executableKey]);
-    m_arguments = map[argumentsKey].toString();
-}
-
-QString StdIOSettings::arguments() const
-{
-    return Utils::globalMacroExpander()->expand(m_arguments);
-}
-
-Utils::CommandLine StdIOSettings::command() const
-{
-    return Utils::CommandLine(m_executable, arguments(), Utils::CommandLine::Raw);
+    return CommandLine(executable(), arguments(), CommandLine::Raw);
 }
 
 BaseClientInterface *StdIOSettings::createInterface(BuildConfiguration *bc) const
@@ -899,20 +894,18 @@ static QString startupBehaviorString(BaseSettings::StartBehavior behavior)
     return {};
 }
 
-BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *parent,
-                                       Layouting::LayoutModifier additionalItems)
+BaseSettingsWidget::BaseSettingsWidget(
+    const BaseSettings *settings, QWidget *parent, Layouting::LayoutModifier additionalItems)
     : QWidget(parent)
-    , m_name(new QLineEdit(settings->m_name, this))
     , m_mimeTypes(new QLabel(settings->m_languageFilter.mimeTypes.join(filterSeparator), this))
     , m_filePattern(
           new QLineEdit(settings->m_languageFilter.filePattern.join(filterSeparator), this))
     , m_startupBehavior(new QComboBox)
-    , m_initializationOptions(new Utils::FancyLineEdit(this))
+    , m_initializationOptions(new FancyLineEdit(this))
 {
     using namespace Layouting;
 
-    auto chooser = new Utils::VariableChooser(this);
-    chooser->addSupportedWidget(m_name);
+    auto chooser = new VariableChooser(this);
     chooser->addSupportedWidget(m_initializationOptions);
 
     auto addMimeTypeButton = new QPushButton(Tr::tr("Set MIME Types..."), this);
@@ -950,7 +943,7 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
 
     // clang-format off
     auto form = Form {
-        Tr::tr("Name:"), m_name, br,
+        settings->name, br,
         Tr::tr("Language:"), Row { m_mimeTypes, st, addMimeTypeButton }, br,
         Tr::tr("File pattern:"), m_filePattern, br,
         Tr::tr("Startup behavior:"), m_startupBehavior, br,
@@ -965,11 +958,6 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
     // clang-format on
 }
 
-QString BaseSettingsWidget::name() const
-{
-    return m_name->text();
-}
-
 LanguageFilter BaseSettingsWidget::filter() const
 {
     return {m_mimeTypes->text().split(filterSeparator, Qt::SkipEmptyParts),
@@ -1030,7 +1018,7 @@ public:
     {
         setWindowTitle(Tr::tr("Select MIME Types"));
         auto mainLayout = new QVBoxLayout;
-        auto filter = new Utils::FancyLineEdit(this);
+        auto filter = new FancyLineEdit(this);
         filter->setFiltering(true);
         mainLayout->addWidget(filter);
         auto listView = new QListView(this);
@@ -1043,8 +1031,8 @@ public:
         connect(buttons, &QDialogButtonBox::accepted, this, &QDialog::accept);
         connect(buttons, &QDialogButtonBox::rejected, this, &QDialog::reject);
         auto proxy = new QSortFilterProxyModel(this);
-        m_mimeTypeModel = new MimeTypeModel(Utils::transform(Utils::allMimeTypes(),
-                                                             &Utils::MimeType::name), this);
+        m_mimeTypeModel
+            = new MimeTypeModel(Utils::transform(Utils::allMimeTypes(), &MimeType::name), this);
         m_mimeTypeModel->m_selectedMimeTypes = selectedMimeTypes;
         proxy->setSourceModel(m_mimeTypeModel);
         proxy->sort(0);
@@ -1080,36 +1068,15 @@ void BaseSettingsWidget::showAddMimeTypeDialog()
 
 StdIOSettingsWidget::StdIOSettingsWidget(const StdIOSettings *settings, QWidget *parent)
     : BaseSettingsWidget(settings, parent)
-    , m_executable(new Utils::PathChooser(this))
-    , m_arguments(new QLineEdit(settings->m_arguments, this))
 {
-    using namespace Layouting;
-
-    m_executable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-    m_executable->setFilePath(settings->m_executable);
-
-    auto mainLayout = qobject_cast<QFormLayout *>(layout());
-    QTC_ASSERT(mainLayout, return);
-    int row = mainLayout->rowCount();
-
-    mainLayout->insertRow(row++, Tr::tr("Executable:"), m_executable);
-    mainLayout->insertRow(row++, Tr::tr("Arguments:"), m_arguments);
-
-    auto chooser = new Utils::VariableChooser(this);
-    chooser->addSupportedWidget(m_arguments);
+    Layouting::Layout l(layout());
+    settings->executable.addToLayout(l);
+    l.flush();
+    settings->arguments.addToLayout(l);
+    l.flush();
 }
 
-Utils::FilePath StdIOSettingsWidget::executable() const
-{
-    return m_executable->filePath();
-}
-
-QString StdIOSettingsWidget::arguments() const
-{
-    return m_arguments->text();
-}
-
-bool LanguageFilter::isSupported(const Utils::FilePath &filePath, const QString &mimeTypeName) const
+bool LanguageFilter::isSupported(const FilePath &filePath, const QString &mimeTypeName) const
 {
     if (!mimeTypes.isEmpty()) {
         const MimeType mimeType = Utils::mimeTypeForName(mimeTypeName);
@@ -1122,8 +1089,9 @@ bool LanguageFilter::isSupported(const Utils::FilePath &filePath, const QString
     if (filePattern.isEmpty() && filePath.isEmpty())
         return mimeTypes.isEmpty();
     const QRegularExpression::PatternOptions options
-            = Utils::HostOsInfo::fileNameCaseSensitivity() == Qt::CaseInsensitive
-            ? QRegularExpression::CaseInsensitiveOption : QRegularExpression::NoPatternOption;
+        = HostOsInfo::fileNameCaseSensitivity() == Qt::CaseInsensitive
+              ? QRegularExpression::CaseInsensitiveOption
+              : QRegularExpression::NoPatternOption;
     auto regexps = Utils::transform(filePattern, [&options](const QString &pattern){
         return QRegularExpression(QRegularExpression::wildcardToRegularExpression(pattern),
                                   options);
@@ -1149,10 +1117,9 @@ bool LanguageFilter::operator!=(const LanguageFilter &other) const
     return this->filePattern != other.filePattern || this->mimeTypes != other.mimeTypes;
 }
 
-TextEditor::BaseTextEditor *createJsonEditor(QObject *parent)
+BaseTextEditor *createJsonEditor(QObject *parent)
 {
-    using namespace TextEditor;
-    using namespace Utils::Text;
+    using namespace Text;
     BaseTextEditor *textEditor = nullptr;
     for (Core::IEditorFactory *factory : Core::IEditorFactory::preferredEditorFactories("foo.json")) {
         Core::IEditor *editor = factory->createEditor();
@@ -1169,8 +1136,8 @@ TextEditor::BaseTextEditor *createJsonEditor(QObject *parent)
     widget->setLineNumbersVisible(false);
     widget->setRevisionsVisible(false);
     widget->setCodeFoldingSupported(false);
-    QObject::connect(document, &TextDocument::contentsChanged, widget, [document](){
-        const Utils::Id jsonMarkId("LanguageClient.JsonTextMarkId");
+    QObject::connect(document, &TextDocument::contentsChanged, widget, [document]() {
+        const Id jsonMarkId("LanguageClient.JsonTextMarkId");
         const TextMarks marks = document->marks();
         for (TextMark *mark : marks) {
             if (mark->category().id == jsonMarkId)
@@ -1186,12 +1153,11 @@ TextEditor::BaseTextEditor *createJsonEditor(QObject *parent)
         const Position pos = Position::fromPositionInDocument(document->document(), error.offset);
         if (!pos.isValid())
             return;
-        auto mark = new TextMark(Utils::FilePath(),
-                                 pos.line,
-                                 {::LanguageClient::Tr::tr("JSON Error"), jsonMarkId});
+        auto mark = new TextMark(
+            FilePath(), pos.line, {::LanguageClient::Tr::tr("JSON Error"), jsonMarkId});
         mark->setLineAnnotation(error.errorString());
-        mark->setColor(Utils::Theme::CodeModel_Error_TextMarkColor);
-        mark->setIcon(Utils::Icons::CODEMODEL_ERROR.icon());
+        mark->setColor(Theme::CodeModel_Error_TextMarkColor);
+        mark->setIcon(Icons::CODEMODEL_ERROR.icon());
         document->addMark(mark);
     });
     return textEditor;
@@ -1293,7 +1259,7 @@ public:
         setGlobalSettingsId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
         setExpanding(true);
 
-        TextEditor::BaseTextEditor *editor = createJsonEditor(this);
+        BaseTextEditor *editor = createJsonEditor(this);
         editor->document()->setContents(m_settings.json());
 
         auto layout = new QVBoxLayout(this);
@@ -1334,7 +1300,7 @@ public:
                     else if (index == 2)
                         m_settings.disableSetting(id);
                 });
-            settingsLayout->addRow(settings->m_name, comboBox);
+            settingsLayout->addRow(settings->name(), comboBox);
         }
 
         auto group = new QGroupBox(Tr::tr("Workspace Configuration"));
@@ -1345,8 +1311,11 @@ public:
         group->layout()->addWidget(editor->widget());
         layout->addWidget(group);
 
-        connect(editor->editorWidget()->textDocument(), &TextEditor::TextDocument::contentsChanged,
-                this, [this, editor] { m_settings.setJson(editor->document()->contents()); });
+        connect(
+            editor->editorWidget()->textDocument(),
+            &TextDocument::contentsChanged,
+            this,
+            [this, editor] { m_settings.setJson(editor->document()->contents()); });
     }
 
 private:
