@@ -1,40 +1,20 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
-#include <model.h>
-#include <modelnode.h>
+#include "model.h"
+#include "modelnode.h"
 
-#include <QObject>
-#include <QUrl>
-#include <QQmlPropertyMap>
-#include <QQmlComponent>
 #include <QColor>
+#include <QObject>
 #include <QPoint>
-#include <QMouseEvent>
+#include <QPointer>
+#include <QQmlComponent>
+#include <QQmlPropertyMap>
+#include <QUrl>
+
+QT_FORWARD_DECLARE_CLASS(QQuickWidget)
 
 namespace QmlDesigner {
 
@@ -42,7 +22,6 @@ class PropertyEditorContextObject : public QObject
 {
     Q_OBJECT
 
-    Q_PROPERTY(QUrl globalBaseUrl READ globalBaseUrl WRITE setGlobalBaseUrl NOTIFY globalBaseUrlChanged)
     Q_PROPERTY(QUrl specificsUrl READ specificsUrl WRITE setSpecificsUrl NOTIFY specificsUrlChanged)
 
     Q_PROPERTY(QString specificQmlData READ specificQmlData WRITE setSpecificQmlData NOTIFY specificQmlDataChanged)
@@ -57,18 +36,32 @@ class PropertyEditorContextObject : public QObject
     Q_PROPERTY(int majorQtQuickVersion READ majorQtQuickVersion WRITE setMajorQtQuickVersion NOTIFY majorQtQuickVersionChanged)
     Q_PROPERTY(int minorQtQuickVersion READ minorQtQuickVersion WRITE setMinorQtQuickVersion NOTIFY minorQtQuickVersionChanged)
 
+    Q_PROPERTY(QString activeDragSuffix READ activeDragSuffix NOTIFY activeDragSuffixChanged)
+
     Q_PROPERTY(bool hasAliasExport READ hasAliasExport NOTIFY hasAliasExportChanged)
 
     Q_PROPERTY(bool hasActiveTimeline READ hasActiveTimeline NOTIFY hasActiveTimelineChanged)
 
-    Q_PROPERTY(QQmlPropertyMap* backendValues READ backendValues WRITE setBackendValues NOTIFY backendValuesChanged)
+    Q_PROPERTY(QQmlPropertyMap *backendValues READ backendValues WRITE setBackendValues NOTIFY backendValuesChanged)
 
-    Q_PROPERTY(QQmlComponent* specificQmlComponent READ specificQmlComponent NOTIFY specificQmlComponentChanged)
+    Q_PROPERTY(QQmlComponent *specificQmlComponent READ specificQmlComponent NOTIFY specificQmlComponentChanged)
+
+    Q_PROPERTY(bool hasMultiSelection READ hasMultiSelection WRITE setHasMultiSelection NOTIFY
+                   hasMultiSelectionChanged)
+
+    Q_PROPERTY(bool isSelectionLocked READ isSelectionLocked WRITE setIsSelectionLocked NOTIFY isSelectionLockedChanged)
+
+    Q_PROPERTY(bool insightEnabled MEMBER m_insightEnabled NOTIFY insightEnabledChanged)
+    Q_PROPERTY(QStringList insightCategories MEMBER m_insightCategories NOTIFY insightCategoriesChanged)
+
+    Q_PROPERTY(bool hasQuick3DImport READ hasQuick3DImport NOTIFY hasQuick3DImportChanged)
+    Q_PROPERTY(bool hasMaterialLibrary READ hasMaterialLibrary NOTIFY hasMaterialLibraryChanged)
+    Q_PROPERTY(bool isQt6Project READ isQt6Project NOTIFY isQt6ProjectChanged)
+    Q_PROPERTY(bool has3DModelSelected READ has3DModelSelected NOTIFY has3DModelSelectedChanged)
 
 public:
     PropertyEditorContextObject(QObject *parent = nullptr);
 
-    QUrl globalBaseUrl() const {return m_globalBaseUrl; }
     QUrl specificsUrl() const {return m_specificsUrl; }
     QString specificQmlData() const {return m_specificQmlData; }
     QString stateName() const {return m_stateName; }
@@ -77,7 +70,7 @@ public:
     bool isBaseState() const { return m_isBaseState; }
     bool selectionChanged() const { return m_selectionChanged; }
 
-    QQmlPropertyMap* backendValues() const { return m_backendValues; }
+    QQmlPropertyMap *backendValues() const { return m_backendValues; }
 
     Q_INVOKABLE QString convertColorToString(const QVariant &color);
     Q_INVOKABLE QColor colorFromString(const QString &colorString);
@@ -85,7 +78,7 @@ public:
 
     Q_INVOKABLE QStringList autoComplete(const QString &text, int pos, bool explicitComplete, bool filter);
 
-    Q_INVOKABLE void toogleExportAlias();
+    Q_INVOKABLE void toggleExportAlias();
 
     Q_INVOKABLE void goIntoComponent();
 
@@ -96,12 +89,30 @@ public:
     Q_INVOKABLE void restoreCursor();
     Q_INVOKABLE void holdCursorInPlace();
 
+    Q_INVOKABLE int devicePixelRatio();
+
     Q_INVOKABLE QStringList styleNamesForFamily(const QString &family);
 
     Q_INVOKABLE QStringList allStatesForId(const QString &id);
 
     Q_INVOKABLE bool isBlocked(const QString &propName) const;
 
+    Q_INVOKABLE void verifyInsightImport();
+
+    Q_INVOKABLE QRect screenRect() const;
+    Q_INVOKABLE QPoint globalPos(const QPoint &point) const;
+
+    Q_INVOKABLE void handleToolBarAction(int action);
+
+    Q_INVOKABLE void saveExpandedState(const QString &sectionName, bool expanded);
+    Q_INVOKABLE bool loadExpandedState(const QString &sectionName, bool defaultValue) const;
+
+    enum ToolBarAction { SelectionLock, SelectionUnlock };
+    Q_ENUM(ToolBarAction)
+
+    QString activeDragSuffix() const;
+    void setActiveDragSuffix(const QString &suffix);
+
     int majorVersion() const;
     int majorQtQuickVersion() const;
     int minorQtQuickVersion() const;
@@ -119,8 +130,32 @@ public:
 
     bool hasAliasExport() const { return m_aliasExport; }
 
+    bool hasMultiSelection() const;
+    void setHasMultiSelection(bool);
+
+    void setInsightEnabled(bool value);
+    void setInsightCategories(const QStringList &categories);
+
+    bool hasQuick3DImport() const;
+    void setHasQuick3DImport(bool value);
+
+    bool hasMaterialLibrary() const;
+    void setHasMaterialLibrary(bool value);
+
+    bool isQt6Project() const;
+    void setIsQt6Project(bool value);
+
+    bool has3DModelSelected() const;
+    void setHas3DModelSelected(bool value);
+
+    void setEditorNodes(const ModelNodes &nodes);
+
+    void setIsSelectionLocked(bool lock);
+    bool isSelectionLocked() const;
+
+    void setQuickWidget(QQuickWidget *newQuickWidget);
+
 signals:
-    void globalBaseUrlChanged();
     void specificsUrlChanged();
     void specificQmlDataChanged();
     void stateNameChanged();
@@ -135,9 +170,19 @@ signals:
     void specificQmlComponentChanged();
     void hasAliasExportChanged();
     void hasActiveTimelineChanged();
+    void activeDragSuffixChanged();
+    void hasMultiSelectionChanged();
+    void hasQuick3DImportChanged();
+    void hasMaterialLibraryChanged();
+    void has3DModelSelectedChanged();
+    void isQt6ProjectChanged();
+    void isSelectionLockedChanged();
+
+    void insightEnabledChanged();
+    void insightCategoriesChanged();
+    void toolBarAction(int action);
 
 public slots:
-     void setGlobalBaseUrl(const QUrl &newBaseUrl);
 
      void setSpecificsUrl(const QUrl &newSpecificsUrl);
 
@@ -151,7 +196,7 @@ public slots:
 
      void setSelectionChanged(bool newSelectionChanged);
 
-     void setBackendValues(QQmlPropertyMap* newBackendValues);
+     void setBackendValues(QQmlPropertyMap *newBackendValues);
 
      void setModel(Model *model);
 
@@ -159,7 +204,6 @@ public slots:
      void setHasAliasExport(bool hasAliasExport);
 
 private:
-    QUrl m_globalBaseUrl;
     QUrl m_specificsUrl;
 
     QString m_specificQmlData;
@@ -169,22 +213,41 @@ private:
     bool m_isBaseState;
     bool m_selectionChanged;
 
-    QQmlPropertyMap* m_backendValues;
+    QQmlPropertyMap *m_backendValues;
 
     int m_majorVersion = 1;
     int m_minorVersion = 1;
     int m_majorQtQuickVersion = 1;
     int m_minorQtQuickVersion = -1;
+
+    bool m_hasQuick3DImport = false;
+    bool m_hasMaterialLibrary = false;
+    bool m_has3DModelSelected = false;
+    bool m_isQt6Project = false;
+
     QQmlComponent *m_qmlComponent;
     QQmlContext *m_qmlContext;
+    QPointer<QQuickWidget> m_quickWidget;
 
     QPoint m_lastPos;
 
-    Model *m_model = nullptr;
+    QPointer<Model> m_model;
 
     bool m_aliasExport = false;
 
     bool m_setHasActiveTimeline = false;
+
+    QString m_activeDragSuffix;
+
+    bool m_hasMultiSelection = false;
+    bool m_isSelectionLocked = false;
+
+    bool m_insightEnabled = false;
+    QStringList m_insightCategories;
+
+    ModelNodes m_editorNodes; // Nodes that are being edited by PropertyEditor
+
+    inline static QHash<QString, bool> s_expandedStateHash;
 };
 
 class EasingCurveEditor : public QObject
