@@ -15,17 +15,17 @@ const char MakeInstallStepId[] = "RemoteLinux.MakeInstall";
 const char TarPackageCreationStepId[]  = "MaemoTarPackageCreationStep";
 const char TarPackageFilePathId[] = "TarPackageFilePath";
 const char TarPackageDeployStepId[] = "MaemoUploadAndInstallTarPackageStep";
-const char RsyncDeployStepId[] = "RemoteLinux.RsyncDeployStep";
+const char GenericDeployStepId[] = "RemoteLinux.RsyncDeployStep";
 const char CustomCommandDeployStepId[] = "RemoteLinux.GenericRemoteLinuxCustomCommandDeploymentStep";
 const char KillAppStepId[] = "RemoteLinux.KillAppStep";
 
-const char SupportsRSync[] = "RemoteLinux.SupportsRSync";
-const char SupportsSftp[] = "RemoteLinux.SupportsSftp";
-const char SourceProfile[] = "RemoteLinux.SourceProfile";
-const char LinkDevice[] = "RemoteLinux.LinkDevice";
+const char SshForwardPort[] = "RemoteLinux.SshForwardPort";
+const char DisableSharing[] = "RemoteLinux.DisableSharing";
 
 const char RunConfigId[] = "RemoteLinuxRunConfiguration:";
 const char CustomRunConfigId[] = "RemoteLinux.CustomRunConfig";
 
+const char ExecutionType[] = "RemoteLinux.ExecutionType";
+
 } // Constants
 } // RemoteLinux
