@@ -1,36 +1,46 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
-** This file is part of the Qt Build Suite.
+** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 #include "commandlinefrontend.h"
 
 #include "application.h"
 #include "consoleprogressobserver.h"
+#include "session.h"
 #include "status.h"
 #include "parser/commandlineoption.h"
 #include "../shared/logging/consolelogger.h"
@@ -40,13 +50,13 @@
 #include <logging/translator.h>
 #include <tools/qbsassert.h>
 #include <tools/projectgeneratormanager.h>
+#include <tools/qttools.h>
 #include <tools/shellutils.h>
 
-#include <QDir>
-#include <QFile>
-#include <QMetaObject>
-#include <QProcessEnvironment>
-#include <QTimer>
+#include <QtCore/qdir.h>
+#include <QtCore/qfile.h>
+#include <QtCore/qprocess.h>
+#include <QtCore/qtimer.h>
 
 #include <cstdlib>
 #include <cstdio>
@@ -59,7 +69,7 @@ CommandLineFrontend::CommandLineFrontend(const CommandLineParser &parser, Settin
     : QObject(parent)
     , m_parser(parser)
     , m_settings(settings)
-    , m_observer(0)
+    , m_observer(nullptr)
     , m_cancelStatus(CancelStatusNone)
     , m_cancelTimer(new QTimer(this))
 {
@@ -68,6 +78,7 @@ CommandLineFrontend::CommandLineFrontend(const CommandLineParser &parser, Settin
 CommandLineFrontend::~CommandLineFrontend()
 {
     m_cancelTimer->stop();
+    delete m_observer;
 }
 
 // Called from interrupt handler. Don't do anything non-trivial here.
@@ -84,11 +95,11 @@ void CommandLineFrontend::checkCancelStatus()
     case CancelStatusRequested:
         m_cancelStatus = CancelStatusCanceling;
         m_cancelTimer->stop();
-        if (m_resolveJobs.isEmpty() && m_buildJobs.isEmpty())
+        if (m_resolveJobs.empty() && m_buildJobs.empty())
             std::exit(EXIT_FAILURE);
-        foreach (AbstractJob * const job, m_resolveJobs)
+        for (AbstractJob * const job : std::as_const(m_resolveJobs))
             job->cancel();
-        foreach (AbstractJob * const job, m_buildJobs)
+        for (AbstractJob * const job : std::as_const(m_buildJobs))
             job->cancel();
         break;
     case CancelStatusCanceling:
@@ -103,61 +114,64 @@ void CommandLineFrontend::start()
         switch (m_parser.command()) {
         case RunCommandType:
         case ShellCommandType:
-            if (m_parser.products().count() > 1) {
+            if (m_parser.products().size() > 1) {
                 throw ErrorInfo(Tr::tr("Invalid use of command '%1': Cannot use more than one "
                                    "product.\nUsage: %2")
                             .arg(m_parser.commandName(), m_parser.commandDescription()));
             }
-            // Fall-through intended.
+            Q_FALLTHROUGH();
         case StatusCommandType:
         case InstallCommandType:
         case DumpNodesTreeCommandType:
-            if (m_parser.buildConfigurations().count() > 1) {
+        case ListProductsCommandType:
+            if (m_parser.buildConfigurations().size() > 1) {
                 QString error = Tr::tr("Invalid use of command '%1': There can be only one "
                                "build configuration.\n").arg(m_parser.commandName());
                 error += Tr::tr("Usage: %1").arg(m_parser.commandDescription());
                 throw ErrorInfo(error);
             }
             break;
+        case SessionCommandType: {
+            startSession();
+            return;
+        }
         default:
             break;
         }
 
         if (m_parser.showVersion()) {
-            puts(QBS_VERSION);
+            std::puts(QBS_VERSION);
             qApp->exit(EXIT_SUCCESS);
             return;
         }
         if (m_parser.showProgress())
             m_observer = new ConsoleProgressObserver;
         SetupProjectParameters params;
+        params.setEnvironment(QProcessEnvironment::systemEnvironment());
         params.setProjectFilePath(m_parser.projectFilePath());
-        params.setIgnoreDifferentProjectFilePath(m_parser.force());
         params.setDryRun(m_parser.dryRun());
+        params.setForceProbeExecution(m_parser.forceProbesExecution());
+        params.setWaitLockBuildGraph(m_parser.waitLockBuildGraph());
         params.setLogElapsedTime(m_parser.logTime());
-        params.setSettingsDirectory(m_settings->baseDirectoy());
-        if (!m_parser.buildBeforeInstalling() || m_parser.command() == DumpNodesTreeCommandType)
+        params.setSettingsDirectory(m_settings->baseDirectory());
+        params.setOverrideBuildGraphData(m_parser.command() == ResolveCommandType);
+        params.setPropertyCheckingMode(ErrorHandlingMode::Strict);
+        params.setDeprecationWarningMode(m_parser.deprecationWarningMode());
+        if (!m_parser.buildBeforeInstalling() || !m_parser.commandCanResolve())
             params.setRestoreBehavior(SetupProjectParameters::RestoreOnly);
-        foreach (const QVariantMap &buildConfig, m_parser.buildConfigurations()) {
+        const auto buildConfigs = m_parser.buildConfigurations();
+        for (const QVariantMap &buildConfig : buildConfigs) {
             QVariantMap userConfig = buildConfig;
-            const QString buildVariantKey = QLatin1String("qbs.buildVariant");
-            const QString profileKey = QLatin1String("qbs.profile");
-            const QString installRootKey = QLatin1String("qbs.installRoot");
+            const QString configurationKey = QStringLiteral("qbs.configurationName");
+            const QString profileKey = QStringLiteral("qbs.profile");
+            const QString installRootKey = QStringLiteral("qbs.installRoot");
             QString installRoot = userConfig.value(installRootKey).toString();
             if (!installRoot.isEmpty() && QFileInfo(installRoot).isRelative()) {
                 installRoot.prepend(QLatin1Char('/')).prepend(QDir::currentPath());
                 userConfig.insert(installRootKey, installRoot);
             }
-            const QString buildVariant = userConfig.take(buildVariantKey).toString();
-            QString profileName = userConfig.take(profileKey).toString();
-            if (profileName.isEmpty())
-                profileName = m_settings->defaultProfile();
-            if (profileName.isEmpty()) {
-                ErrorInfo error(Tr::tr("No profile specified and no default profile exists."));
-                error.append(Tr::tr("To set a default profile, run "
-                                    "'qbs config defaultProfile <profile name>'."));
-                throw error;
-            }
+            const QString configurationName = userConfig.take(configurationKey).toString();
+            const QString profileName = userConfig.take(profileKey).toString();
             const Preferences prefs(m_settings);
             params.setSearchPaths(prefs.searchPaths(QDir::cleanPath(QCoreApplication::applicationDirPath()
                     + QLatin1String("/" QBS_RELATIVE_SEARCH_PATH))));
@@ -166,35 +180,36 @@ void CommandLineFrontend::start()
             params.setLibexecPath(QDir::cleanPath(QCoreApplication::applicationDirPath()
                     + QLatin1String("/" QBS_RELATIVE_LIBEXEC_PATH)));
             params.setTopLevelProfile(profileName);
-            params.setBuildVariant(buildVariant);
+            params.setConfigurationName(configurationName);
             params.setBuildRoot(buildDirectory(profileName));
             params.setOverriddenValues(userConfig);
+            params.setMaxJobCount(m_parser.jobCount(profileName));
             SetupProjectJob * const job = Project().setupProject(params,
                     ConsoleLogger::instance().logSink(), this);
             connectJob(job);
-            m_resolveJobs << job;
+            m_resolveJobs.push_back(job);
         }
 
         /*
          * Progress reporting on the terminal gets a bit tricky when resolving several projects
          * concurrently, since we cannot show multiple progress bars at the same time. Instead,
          * we just set the total effort to the number of projects and increase the progress
-         * every time one of them finishes, ingoring the progress reports from the jobs themselves.
+         * every time one of them finishes, ignoring the progress reports from the jobs themselves.
          * (Yes, that does mean it will take disproportionately long for the first progress
          * notification to arrive.)
          */
         if (m_parser.showProgress() && resolvingMultipleProjects())
-            m_observer->initialize(tr("Setting up projects"), m_resolveJobs.count());
+            m_observer->initialize(tr("Setting up projects"), m_resolveJobs.size());
 
         // Check every two seconds whether we received a cancel request. This value has been
         // experimentally found to be acceptable.
         // Note that this polling approach is not problematic here, since we are doing work anyway,
         // so there's no danger of waking up the processor for no reason.
-        connect(m_cancelTimer, SIGNAL(timeout()), SLOT(checkCancelStatus()));
+        connect(m_cancelTimer, &QTimer::timeout, this, &CommandLineFrontend::checkCancelStatus);
         m_cancelTimer->start(2000);
     } catch (const ErrorInfo &error) {
         qbsError() << error.toString();
-        if (m_buildJobs.isEmpty() && m_resolveJobs.isEmpty()) {
+        if (m_buildJobs.empty() && m_resolveJobs.empty()) {
             qApp->exit(EXIT_FAILURE);
         } else {
             cancel();
@@ -217,17 +232,17 @@ void CommandLineFrontend::handleJobFinished(bool success, AbstractJob *job)
             qbsError() << job->error().toString();
             m_resolveJobs.removeOne(job);
             m_buildJobs.removeOne(job);
-            if (m_resolveJobs.isEmpty() && m_buildJobs.isEmpty()) {
+            if (m_resolveJobs.empty() && m_buildJobs.empty()) {
                 qApp->exit(EXIT_FAILURE);
                 return;
             }
             cancel();
-        } else if (SetupProjectJob * const setupJob = qobject_cast<SetupProjectJob *>(job)) {
+        } else if (const auto setupJob = qobject_cast<SetupProjectJob * const>(job)) {
             m_resolveJobs.removeOne(job);
-            m_projects << setupJob->project();
+            m_projects.push_back(setupJob->project());
             if (m_observer && resolvingMultipleProjects())
                 m_observer->incrementProgressValue();
-            if (m_resolveJobs.isEmpty())
+            if (m_resolveJobs.empty())
                 handleProjectsResolved();
         } else if (qobject_cast<InstallJob *>(job)) {
             if (m_parser.command() == RunCommandType)
@@ -236,7 +251,7 @@ void CommandLineFrontend::handleJobFinished(bool success, AbstractJob *job)
                 qApp->quit();
         } else { // Build or clean.
             m_buildJobs.removeOne(job);
-            if (m_buildJobs.isEmpty()) {
+            if (m_buildJobs.empty()) {
                 switch (m_parser.command()) {
                 case RunCommandType:
                 case InstallCommandType:
@@ -247,7 +262,7 @@ void CommandLineFrontend::handleJobFinished(bool success, AbstractJob *job)
                     // fall through
                 case BuildCommandType:
                 case CleanCommandType:
-                    qApp->quit();
+                    qApp->exit(m_cancelStatus == CancelStatusNone ? EXIT_SUCCESS : EXIT_FAILURE);
                     break;
                 default:
                     Q_ASSERT_X(false, Q_FUNC_INFO, "Missing case in switch statement");
@@ -302,51 +317,64 @@ void CommandLineFrontend::handleTaskProgress(int value, AbstractJob *job)
 
 void CommandLineFrontend::handleProcessResultReport(const qbs::ProcessResult &result)
 {
-    bool hasOutput = !result.stdOut().isEmpty() || !result.stdErr().isEmpty();
+    bool hasOutput = !result.stdOut().empty() || !result.stdErr().empty();
     if (!hasOutput && result.success())
         return;
 
-    (result.success() ? qbsInfo() : qbsError())
-            << shellQuote(QDir::toNativeSeparators(result.executableFilePath()), result.arguments())
-            << (hasOutput ? QString::fromLatin1("\n") : QString())
-            << (result.stdOut().isEmpty() ? QString() : result.stdOut().join(QLatin1Char('\n')))
-            << (result.stdErr().isEmpty() ? QString() : result.stdErr().join(QLatin1Char('\n')));
+    LogWriter w = result.success() ? qbsInfo() : qbsError();
+    w << shellQuote(QDir::toNativeSeparators(result.executableFilePath()), result.arguments())
+      << (hasOutput ? QStringLiteral("\n") : QString())
+      << (result.stdOut().empty() ? QString() : result.stdOut().join(QLatin1Char('\n')));
+    if (!result.stdErr().empty())
+        w << result.stdErr().join(QLatin1Char('\n')) << MessageTag(QStringLiteral("stdErr"));
 }
 
 bool CommandLineFrontend::resolvingMultipleProjects() const
 {
-    return isResolving() && m_resolveJobs.count() + m_projects.count() > 1;
+    return isResolving() && m_resolveJobs.size() + m_projects.size() > 1;
 }
 
 bool CommandLineFrontend::isResolving() const
 {
-    return !m_resolveJobs.isEmpty();
+    return !m_resolveJobs.empty();
 }
 
 bool CommandLineFrontend::isBuilding() const
 {
-    return !m_buildJobs.isEmpty();
+    return !m_buildJobs.empty();
 }
 
 CommandLineFrontend::ProductMap CommandLineFrontend::productsToUse() const
 {
     ProductMap products;
     QStringList productNames;
-    const bool useAll = m_parser.products().isEmpty();
-    foreach (const Project &project, m_projects) {
+    const bool useAll = m_parser.products().empty();
+    for (const Project &project : std::as_const(m_projects)) {
         QList<ProductData> &productList = products[project];
         const ProjectData projectData = project.projectData();
-        foreach (const ProductData &product, projectData.allProducts()) {
-            if (useAll || m_parser.products().contains(product.name())) {
-                productList << product;
-                productNames << product.name();
+        for (const ProductData &product : projectData.allProducts()) {
+            productNames << product.fullDisplayName();
+            if (useAll || m_parser.products().contains(product.fullDisplayName())) {
+                productList.push_back(product);
             }
         }
     }
 
-    foreach (const QString &productName, m_parser.products()) {
-        if (!productNames.contains(productName))
-            throw ErrorInfo(Tr::tr("No such product '%1'.").arg(productName));
+    const auto parsedProductNames = m_parser.products();
+    for (const QString &productName : parsedProductNames) {
+        if (!productNames.contains(productName)) {
+            QString msg;
+            if (productNames.size() <= 10) {
+                productNames.sort();
+                const QString available = productNames.join(QLatin1String("', '"));
+                msg = Tr::tr("No such product '%1'. "
+                             "Available products: '%2'").arg(productName, available);
+            } else {
+                msg = Tr::tr("No such product '%1'. Use 'list-products' to see "
+                                  "all available products.").arg(productName);
+            }
+            throw ErrorInfo(msg);
+        }
     }
 
     return products;
@@ -367,10 +395,12 @@ void CommandLineFrontend::handleProjectsResolved()
         qApp->exit(runShell());
         break;
     case StatusCommandType:
-        qApp->exit(printStatus(m_projects.first().projectData()));
+        qApp->exit(printStatus(m_projects.front().projectData()));
         break;
-    case BuildCommandType:
     case GenerateCommandType:
+        checkGeneratorName();
+        Q_FALLTHROUGH();
+    case BuildCommandType:
         build();
         break;
     case InstallCommandType:
@@ -388,23 +418,30 @@ void CommandLineFrontend::handleProjectsResolved()
         dumpNodesTree();
         qApp->quit();
         break;
+    case ListProductsCommandType:
+        listProducts();
+        qApp->quit();
+        break;
     case HelpCommandType:
+    case VersionCommandType:
+    case SessionCommandType:
         Q_ASSERT_X(false, Q_FUNC_INFO, "Impossible.");
     }
 }
 
 void CommandLineFrontend::makeClean()
 {
-    if (m_parser.products().isEmpty()) {
-        foreach (const Project &project, m_projects) {
+    if (m_parser.products().empty()) {
+        for (const Project &project : std::as_const(m_projects)) {
             m_buildJobs << project.cleanAllProducts(m_parser.cleanOptions(project.profile()), this);
         }
     } else {
         const ProductMap &products = productsToUse();
         for (ProductMap::ConstIterator it = products.begin(); it != products.end(); ++it) {
-            m_buildJobs << it.key().cleanSomeProducts(it.value(),
-                                                      m_parser.cleanOptions(it.key().profile()),
-                                                      this);
+            m_buildJobs.push_back(it.key().cleanSomeProducts(
+                                      it.value(),
+                                      m_parser.cleanOptions(it.key().profile()),
+                                      this));
         }
     }
     connectBuildJobs();
@@ -412,16 +449,25 @@ void CommandLineFrontend::makeClean()
 
 int CommandLineFrontend::runShell()
 {
-    const ProductData productToRun = getTheOneRunnableProduct();
-    RunEnvironment runEnvironment = m_projects.first().getRunEnvironment(productToRun,
-            m_parser.installOptions(m_projects.first().profile()),
-            QProcessEnvironment::systemEnvironment(), m_settings);
-    return runEnvironment.runShell();
+    ProductData productToRun;
+    switch (m_parser.products().size()) {
+    case 0: // No specific product, use project-global environment.
+        break;
+    case 1:
+        productToRun = productsToUse().values().front().front();
+        break;
+    default:
+        throw ErrorInfo(Tr::tr("The command '%1' cannot take more than one product."));
+    }
+    RunEnvironment runEnvironment = m_projects.front().getRunEnvironment(productToRun,
+            m_parser.installOptions(m_projects.front().profile()),
+            QProcessEnvironment::systemEnvironment(), QStringList(), m_settings);
+    return runEnvironment.doRunShell();
 }
 
 BuildOptions CommandLineFrontend::buildOptions(const Project &project) const
 {
-    BuildOptions options = m_parser.buildOptions(m_projects.first().profile());
+    BuildOptions options = m_parser.buildOptions(m_projects.front().profile());
     if (options.maxJobCount() <= 0) {
         const QString profileName = project.profile();
         QBS_CHECK(!profileName.isEmpty());
@@ -444,9 +490,22 @@ QString CommandLineFrontend::buildDirectory(const QString &profileName) const
     }
 
     QString projectName(QFileInfo(m_parser.projectFilePath()).baseName());
+    QString originalBuildDir = buildDir;
     buildDir.replace(BuildDirectoryOption::magicProjectString(), projectName);
+    const QString buildDirPlaceHolderMsgTemplate = Tr::tr(
+        "You must provide the path to the project file when using build directory "
+        "placeholder '%1'.");
+    if (buildDir != originalBuildDir && projectName.isEmpty()) {
+        throw ErrorInfo(
+            buildDirPlaceHolderMsgTemplate.arg(BuildDirectoryOption::magicProjectString()));
+    }
     QString projectDir(QFileInfo(m_parser.projectFilePath()).path());
+    originalBuildDir = buildDir;
     buildDir.replace(BuildDirectoryOption::magicProjectDirString(), projectDir);
+    if (buildDir != originalBuildDir && projectDir.isEmpty()) {
+        throw ErrorInfo(
+            buildDirPlaceHolderMsgTemplate.arg(BuildDirectoryOption::magicProjectDirString()));
+    }
     if (!QFileInfo(buildDir).isAbsolute())
         buildDir = QDir::currentPath() + QLatin1Char('/') + buildDir;
     buildDir = QDir::cleanPath(buildDir);
@@ -455,15 +514,16 @@ QString CommandLineFrontend::buildDirectory(const QString &profileName) const
 
 void CommandLineFrontend::build()
 {
-    if (m_parser.products().isEmpty()) {
+    if (m_parser.products().empty()) {
         const Project::ProductSelection productSelection = m_parser.withNonDefaultProducts()
                 ? Project::ProductSelectionWithNonDefault : Project::ProductSelectionDefaultOnly;
-        foreach (const Project &project, m_projects)
+        for (const Project &project : std::as_const(m_projects))
             m_buildJobs << project.buildAllProducts(buildOptions(project), productSelection, this);
     } else {
         const ProductMap &products = productsToUse();
         for (ProductMap::ConstIterator it = products.begin(); it != products.end(); ++it)
-            m_buildJobs << it.key().buildSomeProducts(it.value(), buildOptions(it.key()), this);
+            m_buildJobs.push_back(it.key().buildSomeProducts(it.value(),
+                                                             buildOptions(it.key()), this));
     }
     connectBuildJobs();
 
@@ -473,47 +533,57 @@ void CommandLineFrontend::build()
      * the respective total efforts as they come in. Once all jobs have reported their total
      * efforts, we can start the overall progress report.
      */
-    m_buildEffortsNeeded = m_buildJobs.count();
+    m_buildEffortsNeeded = m_buildJobs.size();
     m_buildEffortsRetrieved = 0;
     m_totalBuildEffort = 0;
     m_currentBuildEffort = 0;
 }
 
-void CommandLineFrontend::generate()
+void CommandLineFrontend::checkGeneratorName()
 {
     const QString generatorName = m_parser.generateOptions().generatorName();
-    QSharedPointer<ProjectGenerator> generator(ProjectGeneratorManager::findGenerator(generatorName));
-    if (!generator) {
-        const QString generatorNames = ProjectGeneratorManager::loadedGeneratorNames()
-                .join(QLatin1String("\n\t"));
-        if (generatorName.isEmpty()) {
-            throw ErrorInfo(Tr::tr("No generator specified. Available generators:\n\t%1")
-                            .arg(generatorNames));
+    m_generator = ProjectGeneratorManager::findGenerator(generatorName);
+    if (m_generator)
+        return;
+    const auto generatorNames = ProjectGeneratorManager::loadedGeneratorNames();
+    if (!generatorNames.empty()) {
+        const QString generatorNamesString = generatorNames.join(QLatin1String("\n\t"));
+        if (!generatorName.isEmpty()) {
+            throw ErrorInfo(Tr::tr("No generator named '%1'. Available generators:\n\t%2")
+                            .arg(generatorName, generatorNamesString));
         }
 
-        throw ErrorInfo(Tr::tr("No generator named '%1'. Available generators:\n\t%2")
-                        .arg(generatorName)
-                        .arg(generatorNames));
+        throw ErrorInfo(Tr::tr("No generator specified. Available generators:\n\t%1")
+                        .arg(generatorNamesString));
     }
 
-    generator->clearProjects();
-    generator->addProjects(m_projects);
-    generator->generate(m_parser.installOptions(m_projects.first().profile()));
+    throw ErrorInfo(Tr::tr("No generator specified or no generators are available."));
+}
+
+void CommandLineFrontend::generate()
+{
+    QBS_CHECK(!!m_generator);
+    const ErrorInfo error = m_generator->generate(m_projects,
+                                                  m_parser.buildConfigurations(),
+                                                  m_parser.installOptions(QString()),
+                                                  m_parser.settingsDir(),
+                                                  ConsoleLogger::instance(m_settings));
+    if (error.hasError())
+        throw error;
 }
 
 int CommandLineFrontend::runTarget()
 {
     const ProductData productToRun = getTheOneRunnableProduct();
-    const QString executableFilePath = m_projects.first().targetExecutable(productToRun,
-            m_parser.installOptions(m_projects.first().profile()));
+    const QString executableFilePath = productToRun.targetExecutable();
     if (executableFilePath.isEmpty()) {
         throw ErrorInfo(Tr::tr("Cannot run: Product '%1' is not an application.")
-                    .arg(productToRun.name()));
+                    .arg(productToRun.fullDisplayName()));
     }
-    RunEnvironment runEnvironment = m_projects.first().getRunEnvironment(productToRun,
-            m_parser.installOptions(m_projects.first().profile()),
-            QProcessEnvironment::systemEnvironment(), m_settings);
-    return runEnvironment.runTarget(executableFilePath, m_parser.runArgs());
+    RunEnvironment runEnvironment = m_projects.front().getRunEnvironment(productToRun,
+            m_parser.installOptions(m_projects.front().profile()),
+            QProcessEnvironment::systemEnvironment(), m_parser.runEnvConfig(), m_settings);
+    return runEnvironment.doRunTarget(executableFilePath, m_parser.runArgs(), m_parser.dryRun());
 }
 
 void CommandLineFrontend::updateTimestamps()
@@ -528,16 +598,33 @@ void CommandLineFrontend::updateTimestamps()
 void CommandLineFrontend::dumpNodesTree()
 {
     QFile stdOut;
-    stdOut.open(stdout, QIODevice::WriteOnly);
-    const ErrorInfo error = m_projects.first().dumpNodesTree(stdOut, productsToUse()
-                                                             .value(m_projects.first()));
+    if (!stdOut.open(stdout, QIODevice::WriteOnly))
+        throw ErrorInfo(Tr::tr("Cannot write to standard output."));
+    const ErrorInfo error = m_projects.front().dumpNodesTree(stdOut, productsToUse()
+                                                             .value(m_projects.front()));
     if (error.hasError())
         throw error;
 }
 
+void CommandLineFrontend::listProducts()
+{
+    const QList<ProductData> products = productsToUse().constBegin().value();
+    QStringList output;
+    for (const ProductData &p : products) {
+        QString productInfo = p.fullDisplayName();
+        if (!p.isEnabled())
+            productInfo.append(QLatin1Char(' ')).append(Tr::tr("[disabled]"));
+        else if (!p.properties().value(QStringLiteral("builtByDefault")).toBool())
+            productInfo.append(QLatin1Char(' ')).append(Tr::tr("[not built by default]"));
+        output += productInfo;
+    }
+    output.sort();
+    qbsInfo() << output.join(QLatin1Char('\n'));
+}
+
 void CommandLineFrontend::connectBuildJobs()
 {
-    foreach (AbstractJob * const job, m_buildJobs)
+    for (AbstractJob * const job : std::as_const(m_buildJobs))
         connectBuildJob(job);
 }
 
@@ -545,53 +632,53 @@ void CommandLineFrontend::connectBuildJob(AbstractJob *job)
 {
     connectJob(job);
 
-    BuildJob *bjob = qobject_cast<BuildJob *>(job);
+    const auto bjob = qobject_cast<const BuildJob *>(job);
     if (!bjob)
         return;
 
-    connect(bjob, SIGNAL(reportCommandDescription(QString,QString)),
-            this, SLOT(handleCommandDescriptionReport(QString,QString)));
-    connect(bjob, SIGNAL(reportProcessResult(qbs::ProcessResult)),
-            this, SLOT(handleProcessResultReport(qbs::ProcessResult)));
+    connect(bjob, &BuildJob::reportCommandDescription,
+            this, &CommandLineFrontend::handleCommandDescriptionReport);
+    connect(bjob, &BuildJob::reportProcessResult,
+            this, &CommandLineFrontend::handleProcessResultReport);
 }
 
 void CommandLineFrontend::connectJob(AbstractJob *job)
 {
-    connect(job, SIGNAL(finished(bool,qbs::AbstractJob*)),
-            SLOT(handleJobFinished(bool,qbs::AbstractJob*)));
-    connect(job, SIGNAL(taskStarted(QString,int,qbs::AbstractJob*)),
-            SLOT(handleNewTaskStarted(QString,int)));
-    connect(job, SIGNAL(totalEffortChanged(int,qbs::AbstractJob*)),
-            SLOT(handleTotalEffortChanged(int)));
+    connect(job, &AbstractJob::finished,
+            this, &CommandLineFrontend::handleJobFinished);
+    connect(job, &AbstractJob::taskStarted,
+            this, &CommandLineFrontend::handleNewTaskStarted);
+    connect(job, &AbstractJob::totalEffortChanged,
+            this, &CommandLineFrontend::handleTotalEffortChanged);
     if (m_parser.showProgress()) {
-        connect(job, SIGNAL(taskProgress(int,qbs::AbstractJob*)),
-                SLOT(handleTaskProgress(int,qbs::AbstractJob*)));
+        connect(job, &AbstractJob::taskProgress,
+                this, &CommandLineFrontend::handleTaskProgress);
     }
 }
 
 ProductData CommandLineFrontend::getTheOneRunnableProduct()
 {
-    QBS_CHECK(m_projects.count() == 1); // Has been checked earlier.
+    QBS_CHECK(m_projects.size() == 1); // Has been checked earlier.
 
-    if (m_parser.products().count() == 1) {
-        foreach (const ProductData &p, m_projects.first().projectData().allProducts()) {
-            if (p.name() == m_parser.products().first())
+    if (m_parser.products().size() == 1) {
+        for (const ProductData &p : m_projects.front().projectData().allProducts()) {
+            if (p.fullDisplayName() == m_parser.products().constFirst())
                 return p;
         }
         QBS_CHECK(false);
     }
-    QBS_CHECK(m_parser.products().count() == 0);
+    QBS_CHECK(m_parser.products().isEmpty());
 
     QList<ProductData> runnableProducts;
-    foreach (const ProductData &p, m_projects.first().projectData().allProducts()) {
+    for (const ProductData &p : m_projects.front().projectData().allProducts()) {
         if (p.isRunnable())
-            runnableProducts << p;
+            runnableProducts.push_back(p);
     }
 
-    if (runnableProducts.count() == 1)
-        return runnableProducts.first();
+    if (runnableProducts.size() == 1)
+        return runnableProducts.front();
 
-    if (runnableProducts.isEmpty()) {
+    if (runnableProducts.empty()) {
         throw ErrorInfo(Tr::tr("Cannot execute command '%1': Project has no runnable product.")
                         .arg(m_parser.commandName()));
     }
@@ -599,29 +686,23 @@ ProductData CommandLineFrontend::getTheOneRunnableProduct()
     ErrorInfo error(Tr::tr("Ambiguous use of command '%1': No product given, but project "
                            "has more than one runnable product.").arg(m_parser.commandName()));
     error.append(Tr::tr("Use the '--products' option with one of the following products:"));
-    foreach (const ProductData &p, runnableProducts) {
-        QString productRepr = QLatin1String("\t") + p.name();
-        if (p.profile() != m_projects.first().profile()) {
-            productRepr.append(QLatin1String(" [")).append(p.profile())
-                    .append(QLatin1Char(']'));
-        }
-        error.append(productRepr);
-    }
+    for (const ProductData &p : std::as_const(runnableProducts))
+        error.append(QLatin1String("\t") + p.fullDisplayName());
     throw error;
 }
 
 void CommandLineFrontend::install()
 {
-    Q_ASSERT(m_projects.count() == 1);
-    const Project project = m_projects.first();
+    Q_ASSERT(m_projects.size() == 1);
+    const Project project = m_projects.front();
     InstallJob *installJob;
-    if (m_parser.products().isEmpty()) {
+    if (m_parser.products().empty()) {
         const Project::ProductSelection productSelection = m_parser.withNonDefaultProducts()
                 ? Project::ProductSelectionWithNonDefault : Project::ProductSelectionDefaultOnly;
         installJob = project.installAllProducts(m_parser.installOptions(project.profile()),
                                                 productSelection);
     } else {
-        const Project project = m_projects.first();
+        const Project project = m_projects.front();
         const ProductMap products = productsToUse();
         installJob = project.installSomeProducts(products.value(project),
                                                  m_parser.installOptions(project.profile()));
